# Main flow for Synopsys fusion compiler 

set REPORT_DIR $env(SOCLABS_SRAM_CHIPLET_DIR)/imp/ASIC/SRAM_CHIPLET/reports
set LOG_DIR $env(SOCLABS_SRAM_CHIPLET_DIR)/imp/ASIC/SRAM_CHIPLET/logs

# Design setup: read libraries and RTL 
redirect -tee -file $LOG_DIR/01_design_setup.log {source ./design_setup.tcl}

# Floorplan setup
redirect -tee -file $LOG_DIR/02_init_floorplan.log {initialize_floorplan -control_type die -use_site_row -side_length {2171 2640} -core_offset {160}}
redirect -tee -file $LOG_DIR/03_floorplan.log {source ./floorplan/fp.tcl}



source ./io_plan.tcl
save_lib sram_chiplet.dlib

# Read Constraints
redirect -tee -file $LOG_DIR/04_constraints.log {read_sdc ../constraints/sram_chiplet.sdc}

# Power Plan
load_upf ../sram_chiplet.upf
redirect -tee -file $LOG_DIR/05_power_plan.log {source ./power_plan.tcl}

# Route RDL
place_io
route_rdl_flip_chip -layers AP

create_io_filler_cells -io_guides [get_io_guides {main_io.top main_io.left main_io.right main_io.bottom}] \
	-reference_cells {PFILLER20_G PFILLER10_G PFILLER5_G PFILLER1_G PFILLER05_G PFILLER0005_G} \
	-prefix IOFILL

# Init coarse placement
redirect -tee -file $LOG_DIR/06_init_placement.log {source ./init_placement.tcl}
current_corner default
set_operating_conditions -max_library cln28ht -max ffg_cbestt_min_0p99v_m40c -min_library cln28ht -min ssg_cworstt_max_0p81v_125c
# CHECK THE TIMING ANALYSIS IN THE GUI


# Physical aware synthesis
redirect -tee -file $LOG_DIR/07_compile.log {compile_fusion}
set_operating_conditions -max_library cln28ht -max ffg_cbestt_min_0p99v_m40c -min_library cln28ht -min ssg_cworstt_max_0p81v_125c
save_lib sram_chiplet.dlib
redirect -tee -file $REPORT_DIR/timing_01_compile.rep {report_timing}


# Optimise Placement
redirect -tee -file $LOG_DIR/09_place_opt.log {place_opt}
save_lib sram_chiplet.dlib
redirect -tee -file $REPORT_DIR/timing_02_opt_place.rep {report_timing}

# CTS
redirect -tee -file $LOG_DIR/08_CTS.log {synthesize_clock_trees -clock [get_clocks]}
save_lib sram_chiplet.dlib
redirect -tee -file $REPORT_DIR/timing_03_CTS.rep {report_timing}

# Clock Optimisation
redirect -tee -file $LOG_DIR/10_clock_opt.log {clock_opt}
save_lib sram_chiplet.dlib
redirect -tee -file $REPORT_DIR/timing_04_clock_opt.rep {report_timing}

create_stdcell_fillers -lib_cells {FILL128_A12PP140ZTS_C35 FILL16_A12PP140ZTS_C35 FILL1_A12PP140ZTS_C35 FILL2_A12PP140ZTS_C35 FILL32_A12PP140ZTS_C35 FILL3_A12PP140ZTS_C35 FILL4_A12PP140ZTS_C35}

# Route
redirect -tee -file $LOG_DIR/11_route.log {route_auto}
save_lib sram_chiplet.dlib
redirect -tee -file $REPORT_DIR/timing_05_route.rep {report_timing}

# Route optimisation
redirect -tee -file $LOG_DIR/12_route_optimisation.log {optimize_routes}
save_lib sram_chiplet.dlib
redirect -tee -file $REPORT_DIR/timing_06_route_opt.rep {report_timing}

## Paths Please Edit for your system
set cln28ht_tech_path           /home/dwn1c21/SoC-Labs/phys_ip/arm/tsmc/cln28ht/arm_tech/r1p0
set standard_cell_base_path     /home/dwn1c21/SoC-Labs/phys_ip/arm/tsmc/cln28ht/sc12mcpp140z_base_svt_c35/r2p0
set pmk_base_path               /home/dwn1c21/SoC-Labs/phys_ip/arm/tsmc/cln28ht/sc12mcpp140z_pmk_svt_c35/r1p0
set ret_base_path               /home/dwn1c21/SoC-Labs/phys_ip/arm/tsmc/cln28ht/sc12mcpp140z_rklo_lvt_svt_c30_c35/r1p0
set Synopsys_PLL_dir /home/dwn1c21/SoC-Labs/Synopsys_ip/IP/PLL/synopsys/dwc_pll3ghz_tsmc28hpcp/1.10a/macro
set Synopsys_TS_dir /home/dwn1c21/SoC-Labs/Synopsys_ip/IP/Southampton_28hpcp_pd_vm_ts_vmps_pvtc/1.01b
set Synopsys_PD_dir /home/dwn1c21/SoC-Labs/Synopsys_ip/IP/Southampton_28hpcp_pd_vm_ts_vmps_pvtc/dwc_sensors_pd_tsmc28hpcp_1.00a/synopsys/dwc_sensors_pd_tsmc28hpcp/1.00a
set Synopsys_VM_dir /home/dwn1c21/SoC-Labs/Synopsys_ip/IP/Southampton_28hpcp_pd_vm_ts_vmps_pvtc/dwc_sensors_vm_shrink_tsmc28hpcp_1.00a/synopsys/dwc_sensors_vm_shrink_tsmc28hpcp/1.00a

set standard_cell_gds_file      $standard_cell_base_path/gds2/sc12mcpp140z_cln28ht_base_svt_c35.gds2
set pmk_gds_file                $pmk_base_path/gds2/sc12mcpp140z_cln28ht_pmk_svt_c35.gds2
set ret_gds_file                $ret_base_path/gds2/sc12mcpp140z_cln28ht_rklo_lvt_svt_c30_c35.gds2
set sram_32k_gds_file           $env(SOCLABS_SRAM_CHIPLET_DIR)/memories/sram_32b_32k/sram_32b_32k.gds2
set sram_16k_gds_file           $env(SOCLABS_SRAM_CHIPLET_DIR)/memories/sram_32b_16k/sram_32b_16k.gds2
set Synopsys_PLL_gds_file 	$Synopsys_PLL_dir/gds/5m4x0z/dwc_z19606ts_ns.gds
set Synopsys_TS_gds_file 	$Synopsys_TS_dir/gdsii/mr74127_v1r1.gds
set Synopsys_PD_gds_file    	$Synopsys_PD_dir/gds/mr74125_v1r2.gds
set Synopsys_VM_gds_file    	$Synopsys_VM_dir/gdsii/mr74140_v1r1.gds

set OUT_DIR $env(SOCLABS_SRAM_CHIPLET_DIR)/imp/ASIC/SRAM_CHIPLET

write_gds -lib_cell_view frame -layer_map /home/dwn1c21/SoC-Labs/phys_ip/arm/tsmc/cln28ht/arm_tech/r1p0/milkyway/1p8m_5x2z_utalrdl/stream_out_layer_map -allow_design_mismatch -long_names -hierarchy all -units 1000 -merge_files [list $standard_cell_gds_file $pmk_gds_file $ret_gds_file $sram_32k_gds_file $sram_16k_gds_file $Synopsys_PLL_gds_file $Synopsys_TS_gds_file $Synopsys_PD_gds_file] $OUT_DIR/sram_chiplet.gds 
write_verilog -top_module_first -hierarchy all \
	-exclude {corner_cells filler_cells flip_chip_pad_cells pad_spacer_cells pg_netlist spare_cells} $OUT_DIR/sram_chiplet_gate.v
write_sdf $OUT_DIR/sram_chiplet_gate.sdf


