#-----------------------------------------------------------------------------
# SRAM Chiplet constraints for ASIC on TSMC 28nm HPC+ node
# A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
#
# Contributors
#
# Daniel Newbrook (d.newbrook@soton.ac.uk)
#
# Copyright (C) 2021-4, SoC Labs (www.soclabs.org)
#-----------------------------------------------------------------------------

set_units -time ns;
set_units -capacitance pF;


#### CLOCK DEFINITION

set REF_CLK "REF_CLK";
set TLX_IN_FWD_CLK "TLX_IN_FWD_CLK"
set TLX_OUT_REV_CLK "TLX_OUT_REV_CLK"
set SYS_CLK "PLL_CLK"


set REF_CLK_PERIOD 4;
set TLX_IN_FWD_CLK_PERIOD 1;
set TLX_OUT_REV_CLK_PERIOD 1;
set SYS_CLK_PERIOD 1;

set REF_CLK_JITTER 0.0274; #from CDCM61001 datasheet 
set REF_CLK_TRANS 0.825;

set TLX_CLK_UNCERTAINTY 0.1;

create_clock -name "$REF_CLK" -period "$REF_CLK_PERIOD"  [get_ports PAD_REF_CLK]
set_clock_uncertainty $REF_CLK_JITTER  $REF_CLK  
set_clock_transition $REF_CLK_TRANS [get_clocks $REF_CLK]

create_clock -name "$TLX_IN_FWD_CLK" -period "$TLX_IN_FWD_CLK_PERIOD"  [get_ports PAD_IN_FWD_CLK]
set_clock_uncertainty $TLX_CLK_UNCERTAINTY $TLX_IN_FWD_CLK

create_clock -name "$TLX_OUT_REV_CLK" -period "$TLX_OUT_REV_CLK_PERIOD" [get_ports PAD_OUT_REV_CLK]
set_clock_uncertainty $TLX_CLK_UNCERTAINTY $TLX_OUT_REV_CLK

create_generated_clock -name "$SYS_CLK" -source [get_ports PAD_REF_CLK] -multiply_by 4 [get_pins u_top_sram_chiplet/u_sram_chiplet_apb_subsystem/PLL_CLK1]
create_generated_clock -name "TLX_IN_REV_CLK" -source [get_pins u_top_sram_chiplet/u_sram_chiplet_apb_subsystem/PLL_CLK1] -multiply_by 1 [get_pins u_PAD_IN_REV_CLK/PAD]


set_output_delay 0.1 -clock TLX_IN_REV_CLK [get_ports PAD_TLX_IN_data_rev_0_tdata]
set_output_delay 0.1 -clock TLX_IN_REV_CLK [get_ports PAD_TLX_IN_data_rev_0_tvalid]
set_output_delay 0.1 -clock TLX_IN_REV_CLK [get_ports PAD_TLX_IN_flow_rev_0_tdata]
set_output_delay 0.1 -clock TLX_IN_REV_CLK [get_ports PAD_TLX_IN_flow_rev_0_tvalid]
set_output_delay 0.1 -clock TLX_IN_REV_CLK [get_ports PAD_TLX_IN_flow_fwd_0_tready]
set_output_delay 0.1 -clock TLX_IN_REV_CLK [get_ports PAD_TLX_IN_data_fwd_0_tready]

set_input_delay 0.1 -clock $TLX_IN_FWD_CLK [get_ports PAD_TLX_IN_data_rev_0_tready]
set_input_delay 0.1 -clock $TLX_IN_FWD_CLK [get_ports PAD_TLX_IN_flow_rev_0_tready]
set_input_delay 0.1 -clock $TLX_IN_FWD_CLK [get_ports PAD_TLX_IN_flow_fwd_0_tdata]
set_input_delay 0.1 -clock $TLX_IN_FWD_CLK [get_ports PAD_TLX_IN_flow_fwd_0_tvalid]
set_input_delay 0.1 -clock $TLX_IN_FWD_CLK [get_ports PAD_TLX_IN_data_fwd_0_tdata]
set_input_delay 0.1 -clock $TLX_IN_FWD_CLK [get_ports PAD_TLX_IN_data_fwd_0_tvalid]

set_max_capacitance 3 [all_outputs]
set_max_fanout 10 [all_inputs]



