

module SRAM #(parameter SPLIT_SRAM =0 )(
    input  wire             clk,
    input  wire [18:0]      memaddr,
    input  wire [31:0]      memd,
    output reg [31:0]      memq,
    input  wire             memcen,
    input  wire [3:0]       memwen
);

localparam N_MEMS = 4;
localparam SEL_W = 2;

reg  [N_MEMS:0]   CEN_i; //Active low chip select
wire [31:0]         q_i[0:N_MEMS];
wire [31:0]         wena_i; 
wire                gwen_i;

assign wena_i = {{8{memwen[3]}}, {8{memwen[2]}}, {8{memwen[1]}}, {8{memwen[0]}}};
assign gwen_i = &memwen;
genvar i;
generate for(i=0; i<N_MEMS-1; i = i + 1) begin: g_srams
    sram_32b_32k u_sram_32b_32k(
        .Q(q_i[i]), 
        .CLK(clk), 
        .CEN(CEN_i[i]), 
        .GWEN(gwen_i), 
        .A(memaddr[16:2]), 
        .D(memd), 
        .WEN(wena_i), 
        .STOV(1'b0), 
        .EMA(3'b011), 
        .EMAW(2'b01), 
        .EMAS(1'b0), 
        .RET1N(1'b1)
    );

end endgenerate

generate if (SPLIT_SRAM == 1) begin : gen_split_sram

sram_32b_16k u_sram_32b_16k_0(
    .Q(q_i[N_MEMS-1]), 
    .CLK(clk), 
    .CEN(CEN_i[N_MEMS-1]), 
    .GWEN(gwen_i), 
    .A(memaddr[15:2]), 
    .D(memd), 
    .WEN(wena_i), 
    .STOV(1'b0), 
    .EMA(3'b011), 
    .EMAW(2'b01), 
    .EMAS(1'b0), 
    .RET1N(1'b1)
);
sram_32b_16k u_sram_32b_16k_1(
    .Q(q_i[N_MEMS]), 
    .CLK(clk), 
    .CEN(CEN_i[N_MEMS]), 
    .GWEN(gwen_i), 
    .A(memaddr[15:2]), 
    .D(memd), 
    .WEN(wena_i), 
    .STOV(1'b0), 
    .EMA(3'b011), 
    .EMAW(2'b01), 
    .EMAS(1'b0), 
    .RET1N(1'b1)
);

integer j;
always @(*) begin 
    for(j=0; j<N_MEMS-1; j=j+1) begin
        if(j==memaddr[(SEL_W+17-1):17])
            CEN_i[j] = memcen;
        else
            CEN_i[j] = 1'b1;
    end
    if(memaddr[SEL_W+16:17]=={SEL_W{1'b1}}) begin
        if(memaddr[16]==1'b0) begin
            CEN_i[N_MEMS-1]=memcen;
            CEN_i[N_MEMS]=1'b1;
        end
        else begin
            CEN_i[N_MEMS]=memcen;
            CEN_i[N_MEMS-1]=1'b1;
        end
    end else
        CEN_i[N_MEMS:N_MEMS-1]=2'b11;
    if(memaddr[SEL_W+16:17]=={SEL_W{1'b1}}) begin
        if(memaddr[16]==1'b1) 
            memq=q_i[N_MEMS];
        else
            memq=q_i[N_MEMS-1];
    end else 
        memq = q_i[memaddr[(SEL_W+17-1):17]];
end
end else begin: gen_sram

    sram_32b_32k u_sram_32b_32k_N(
        .Q(q_i[N_MEMS-1]), 
        .CLK(clk), 
        .CEN(CEN_i[N_MEMS-1]), 
        .GWEN(gwen_i), 
        .A(memaddr[16:2]), 
        .D(memd), 
        .WEN(wena_i), 
        .STOV(1'b0), 
        .EMA(3'b011), 
        .EMAW(2'b01), 
        .EMAS(1'b0), 
        .RET1N(1'b1)
    );

integer j;
always @(*) begin 
    for(j=0; j<N_MEMS; j=j+1) begin
        if(j==memaddr[(SEL_W+17-1):17])
            CEN_i[j] = memcen;
        else
            CEN_i[j] = 1'b1;
    end
    memq = q_i[memaddr[(SEL_W+17-1):17]];
end
end endgenerate

endmodule
