//-----------------------------------------------------------------------------
// Expansion Subsystem SRAM Wrapper
// A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
//
// Contributors
//
// Daniel Newbrook (d.newbrook@soton.ac.uk)
// 
// Copyright � 2021-4, SoC Labs (www.soclabs.org)
//-----------------------------------------------------------------------------
// Modules instantiated:
//  sie300_axi5_sram_ctrl_expansion_subsystem
//  SRAM

module SRAM_wrapper #(
    parameter ID_W = 12,
    parameter SPLIT_SRAM = 0
)(
    input  wire             ACLK,
    input  wire             ARESETn,

    input  wire             AWVALID,
    output wire             AWREADY,
    input  wire [ID_W-1:0]       AWID,
    input  wire [31:0]      AWADDR,
    input  wire [7:0]       AWLEN,
    input  wire [2:0]       AWSIZE,
    input  wire [1:0]       AWBURST,
    input  wire             AWLOCK,
    input  wire [2:0]       AWPROT,
    input  wire [3:0]       AWQOS,
    
    input  wire             WVALID,
    output wire             WREADY,
    input  wire [31:0]      WDATA,
    input  wire [3:0]       WSTRB,
    input  wire             WLAST,
    input  wire             WPOISON,

    output wire             BVALID,
    input  wire             BREADY,
    output wire [ID_W-1:0]       BID,
    output wire [1:0]       BRESP,
    
    input  wire             ARVALID,
    output wire             ARREADY,
    input  wire [ID_W-1:0]       ARID,
    input  wire [31:0]      ARADDR,
    input  wire [7:0]       ARLEN,
    input  wire [2:0]       ARSIZE,
    input  wire [1:0]       ARBURST,
    input  wire             ARLOCK,
    input  wire [2:0]       ARPROT,
    input  wire [3:0]       ARQOS,
    
    output wire             RVALID,
    input  wire             RREADY,
    output wire [ID_W-1:0]       RID,
    output wire [31:0]      RDATA,
    output wire [1:0]       RRESP,
    output wire             RLAST,
    output wire             RPOISON,
    input  wire             AWAKEUP,

    input  wire             clk_qreqn,
    output wire             clk_qacceptn,
    output wire             clk_qdeny,
    output wire             clk_qactive,

    input  wire             pwr_qreqn,
    output wire             pwr_qacceptn,
    output wire             pwr_qdeny,
    output wire             pwr_qactive,

    input  wire             ext_gt_qreqn,
    output wire             ext_gt_qacceptn,
    input  wire             cfg_gate_resp

);


wire [18:0]     memaddr;
wire [31:0]    memd;
wire [31:0]    memq;
wire            memcen;
wire [3:0]     memwen;

sie300_axi5_sram_ctrl_sram_chiplet u_SMC(
    .aclk(ACLK),
    .aresetn(ARESETn),
    .awvalid_s(AWVALID),
    .awready_s(AWREADY),
    .awid_s(AWID),
    .awaddr_s(AWADDR[18:0]),
    .awlen_s(AWLEN),
    .awsize_s(AWSIZE),
    .awburst_s(AWBURST),
    .awlock_s(AWLOCK),
    .awprot_s(AWPROT),
    .awqos_s(AWQOS),
    .wvalid_s(WVALID),
    .wready_s(WREADY),
    .wdata_s(WDATA),
    .wstrb_s(WSTRB),
    .wlast_s(WLAST),
    .wpoison_s(WPOISON),
    .bvalid_s(BVALID),
    .bready_s(BREADY),
    .bid_s(BID),
    .bresp_s(BRESP),
    .arvalid_s(ARVALID),
    .arready_s(ARREADY),
    .arid_s(ARID),
    .araddr_s(ARADDR[18:0]),
    .arlen_s(ARLEN),
    .arsize_s(ARSIZE),
    .arburst_s(ARBURST),
    .arlock_s(ARLOCK),
    .arprot_s(ARPROT),
    .arqos_s(ARQOS),
    .rvalid_s(RVALID),
    .rready_s(RREADY),
    .rid_s(RID),
    .rdata_s(RDATA),
    .rresp_s(RRESP),
    .rlast_s(RLAST),
    .rpoison_s(RPOISON),
    .awakeup_s(AWAKEUP),
    .clk_qreqn(clk_qreqn),
    .clk_qacceptn(clk_qacceptn),
    .clk_qdeny(clk_qdeny),
    .clk_qactive(clk_qactive),
    .pwr_qreqn(pwr_qreqn),
    .pwr_qacceptn(pwr_qacceptn),
    .pwr_qdeny(pwr_qdeny),
    .pwr_qactive(pwr_qactive),
    .ext_gt_qreqn(ext_gt_qreqn),
    .ext_gt_qacceptn(ext_gt_qacceptn),
    .cfg_gate_resp(cfg_gate_resp),
    .memaddr(memaddr),
    .memd(memd),
    .memq(memq),
    .memcen(memcen),
    .memwen(memwen)
);

SRAM #(.SPLIT_SRAM(SPLIT_SRAM)) 
    u_SRAM(
    .clk(ACLK),
    .memaddr(memaddr),
    .memd(memd),
    .memq(memq),
    .memcen(memcen),
    .memwen(memwen)
);

endmodule