//-----------------------------------------------------------------------------
// SRAM Chiplet Wrapper
// A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
//
// Contributors
//
// Daniel Newbrook (d.newbrook@soton.ac.uk)
// 
// Copyright � 2021-4, SoC Labs (www.soclabs.org)
//-----------------------------------------------------------------------------
// Modules instantiated:
//  nic400_sram_chiplet
//  nic400_cd_pl_rev_M1_m_tlx_tlx_sram_chiplet
//  nic400_master_pwr_M1_m_tlx_tlx_sram_chiplet
//  sram_chiplet_apb_subsystem
//  SRAM_wrapper

`include "tlx_interfaces.sv"

module top_sram_chiplet #(
    parameter N_ADDR_SEL_BITS = 4
    )(
    // Clock and reset
    input  wire         REF_CLK,
    input  wire         aRESETn,
    input  wire         DL_FWD_RESETn,
    input  wire         IN_FWD_CLK,
    output wire         IN_REV_CLK,
    output wire         OUT_FWD_CLK,
    input  wire         OUT_REV_CLK,
    output wire         OUT_CLK,
    output wire         PLL_LOCK_o,
    input  wire         PLL_LOCK_i,

    // Thin links interface Input
    TLX_AXI_stream.out  TLX_IN_data_rev,
    TLX_AXI_stream.out  TLX_IN_flow_rev,
    TLX_AXI_stream.in   TLX_IN_flow_fwd,
    TLX_AXI_stream.in   TLX_IN_data_fwd,

    // Thin links interface output
    TLX_AXI_stream.in   TLX_OUT_data_rev,
    TLX_AXI_stream.in   TLX_OUT_flow_rev,
    TLX_AXI_stream.out  TLX_OUT_flow_fwd,
    TLX_AXI_stream.out  TLX_OUT_data_fwd,
    
    // Address Select pins
    input  wire [N_ADDR_SEL_BITS-1:0]   addr_sel

    
);
localparam ID_W = 12;
wire SYS_CLK;

// Main Bus Wires
// - SRAM AXI wires
wire [ID_W-1:0]     AWID_AXI_SRAM_0;
wire [31:0]         AWADDR_AXI_SRAM_0;
wire [7:0]          AWLEN_AXI_SRAM_0;
wire [2:0]          AWSIZE_AXI_SRAM_0;
wire [1:0]          AWBURST_AXI_SRAM_0;
wire                AWLOCK_AXI_SRAM_0;
wire [3:0]          AWCACHE_AXI_SRAM_0;
wire [2:0]          AWPROT_AXI_SRAM_0;
wire                AWVALID_AXI_SRAM_0;
wire                AWREADY_AXI_SRAM_0;
wire [31:0]         WDATA_AXI_SRAM_0;
wire [3:0]          WSTRB_AXI_SRAM_0;
wire                WLAST_AXI_SRAM_0;
wire                WVALID_AXI_SRAM_0;
wire                WREADY_AXI_SRAM_0;
wire [ID_W-1:0]     BID_AXI_SRAM_0;
wire [1:0]          BRESP_AXI_SRAM_0;
wire                BVALID_AXI_SRAM_0;
wire                BREADY_AXI_SRAM_0;
wire [ID_W-1:0]     ARID_AXI_SRAM_0;
wire [31:0]         ARADDR_AXI_SRAM_0;
wire [7:0]          ARLEN_AXI_SRAM_0;
wire [2:0]          ARSIZE_AXI_SRAM_0;
wire [1:0]          ARBURST_AXI_SRAM_0;
wire                ARLOCK_AXI_SRAM_0;
wire [3:0]          ARCACHE_AXI_SRAM_0;
wire [2:0]          ARPROT_AXI_SRAM_0;
wire                ARVALID_AXI_SRAM_0;
wire                ARREADY_AXI_SRAM_0;
wire [ID_W-1:0]     RID_AXI_SRAM_0;
wire [31:0]         RDATA_AXI_SRAM_0;
wire [1:0]          RRESP_AXI_SRAM_0;
wire                RLAST_AXI_SRAM_0;
wire                RVALID_AXI_SRAM_0;
wire                RREADY_AXI_SRAM_0;

wire [ID_W-1:0]     AWID_AXI_SRAM_1;
wire [31:0]         AWADDR_AXI_SRAM_1;
wire [7:0]          AWLEN_AXI_SRAM_1;
wire [2:0]          AWSIZE_AXI_SRAM_1;
wire [1:0]          AWBURST_AXI_SRAM_1;
wire                AWLOCK_AXI_SRAM_1;
wire [3:0]          AWCACHE_AXI_SRAM_1;
wire [2:0]          AWPROT_AXI_SRAM_1;
wire                AWVALID_AXI_SRAM_1;
wire                AWREADY_AXI_SRAM_1;
wire [31:0]         WDATA_AXI_SRAM_1;
wire [3:0]          WSTRB_AXI_SRAM_1;
wire                WLAST_AXI_SRAM_1;
wire                WVALID_AXI_SRAM_1;
wire                WREADY_AXI_SRAM_1;
wire [ID_W-1:0]     BID_AXI_SRAM_1;
wire [1:0]          BRESP_AXI_SRAM_1;
wire                BVALID_AXI_SRAM_1;
wire                BREADY_AXI_SRAM_1;
wire [ID_W-1:0]     ARID_AXI_SRAM_1;
wire [31:0]         ARADDR_AXI_SRAM_1;
wire [7:0]          ARLEN_AXI_SRAM_1;
wire [2:0]          ARSIZE_AXI_SRAM_1;
wire [1:0]          ARBURST_AXI_SRAM_1;
wire                ARLOCK_AXI_SRAM_1;
wire [3:0]          ARCACHE_AXI_SRAM_1;
wire [2:0]          ARPROT_AXI_SRAM_1;
wire                ARVALID_AXI_SRAM_1;
wire                ARREADY_AXI_SRAM_1;
wire [ID_W-1:0]     RID_AXI_SRAM_1;
wire [31:0]         RDATA_AXI_SRAM_1;
wire [1:0]          RRESP_AXI_SRAM_1;
wire                RLAST_AXI_SRAM_1;
wire                RVALID_AXI_SRAM_1;
wire                RREADY_AXI_SRAM_1;

wire [ID_W-1:0]     AWID_AXI_SRAM_2;
wire [31:0]         AWADDR_AXI_SRAM_2;
wire [7:0]          AWLEN_AXI_SRAM_2;
wire [2:0]          AWSIZE_AXI_SRAM_2;
wire [1:0]          AWBURST_AXI_SRAM_2;
wire                AWLOCK_AXI_SRAM_2;
wire [3:0]          AWCACHE_AXI_SRAM_2;
wire [2:0]          AWPROT_AXI_SRAM_2;
wire                AWVALID_AXI_SRAM_2;
wire                AWREADY_AXI_SRAM_2;
wire [31:0]         WDATA_AXI_SRAM_2;
wire [3:0]          WSTRB_AXI_SRAM_2;
wire                WLAST_AXI_SRAM_2;
wire                WVALID_AXI_SRAM_2;
wire                WREADY_AXI_SRAM_2;
wire [ID_W-1:0]     BID_AXI_SRAM_2;
wire [1:0]          BRESP_AXI_SRAM_2;
wire                BVALID_AXI_SRAM_2;
wire                BREADY_AXI_SRAM_2;
wire [ID_W-1:0]     ARID_AXI_SRAM_2;
wire [31:0]         ARADDR_AXI_SRAM_2;
wire [7:0]          ARLEN_AXI_SRAM_2;
wire [2:0]          ARSIZE_AXI_SRAM_2;
wire [1:0]          ARBURST_AXI_SRAM_2;
wire                ARLOCK_AXI_SRAM_2;
wire [3:0]          ARCACHE_AXI_SRAM_2;
wire [2:0]          ARPROT_AXI_SRAM_2;
wire                ARVALID_AXI_SRAM_2;
wire                ARREADY_AXI_SRAM_2;
wire [ID_W-1:0]     RID_AXI_SRAM_2;
wire [31:0]         RDATA_AXI_SRAM_2;
wire [1:0]          RRESP_AXI_SRAM_2;
wire                RLAST_AXI_SRAM_2;
wire                RVALID_AXI_SRAM_2;
wire                RREADY_AXI_SRAM_2;

wire [ID_W-1:0]     AWID_AXI_SRAM_3;
wire [31:0]         AWADDR_AXI_SRAM_3;
wire [7:0]          AWLEN_AXI_SRAM_3;
wire [2:0]          AWSIZE_AXI_SRAM_3;
wire [1:0]          AWBURST_AXI_SRAM_3;
wire                AWLOCK_AXI_SRAM_3;
wire [3:0]          AWCACHE_AXI_SRAM_3;
wire [2:0]          AWPROT_AXI_SRAM_3;
wire                AWVALID_AXI_SRAM_3;
wire                AWREADY_AXI_SRAM_3;
wire [31:0]         WDATA_AXI_SRAM_3;
wire [3:0]          WSTRB_AXI_SRAM_3;
wire                WLAST_AXI_SRAM_3;
wire                WVALID_AXI_SRAM_3;
wire                WREADY_AXI_SRAM_3;
wire [ID_W-1:0]     BID_AXI_SRAM_3;
wire [1:0]          BRESP_AXI_SRAM_3;
wire                BVALID_AXI_SRAM_3;
wire                BREADY_AXI_SRAM_3;
wire [ID_W-1:0]     ARID_AXI_SRAM_3;
wire [31:0]         ARADDR_AXI_SRAM_3;
wire [7:0]          ARLEN_AXI_SRAM_3;
wire [2:0]          ARSIZE_AXI_SRAM_3;
wire [1:0]          ARBURST_AXI_SRAM_3;
wire                ARLOCK_AXI_SRAM_3;
wire [3:0]          ARCACHE_AXI_SRAM_3;
wire [2:0]          ARPROT_AXI_SRAM_3;
wire                ARVALID_AXI_SRAM_3;
wire                ARREADY_AXI_SRAM_3;
wire [ID_W-1:0]     RID_AXI_SRAM_3;
wire [31:0]         RDATA_AXI_SRAM_3;
wire [1:0]          RRESP_AXI_SRAM_3;
wire                RLAST_AXI_SRAM_3;
wire                RVALID_AXI_SRAM_3;
wire                RREADY_AXI_SRAM_3;

// - Thin links out wires
wire [ID_W-1:0]  AWID_AXI_TLX_OUT;
wire [31:0] AWADDR_AXI_TLX_OUT;
wire [7:0]  AWLEN_AXI_TLX_OUT;
wire [2:0]  AWSIZE_AXI_TLX_OUT;
wire [1:0]  AWBURST_AXI_TLX_OUT;
wire        AWLOCK_AXI_TLX_OUT;
wire [3:0]  AWCACHE_AXI_TLX_OUT;
wire [2:0]  AWPROT_AXI_TLX_OUT;
wire        AWVALID_AXI_TLX_OUT;
wire        AWREADY_AXI_TLX_OUT;
wire [31:0] WDATA_AXI_TLX_OUT;
wire [3:0]  WSTRB_AXI_TLX_OUT;
wire        WLAST_AXI_TLX_OUT;
wire        WVALID_AXI_TLX_OUT;
wire        WREADY_AXI_TLX_OUT;
wire [ID_W-1:0]  BID_AXI_TLX_OUT;
wire [1:0]  BRESP_AXI_TLX_OUT;
wire        BVALID_AXI_TLX_OUT;
wire        BREADY_AXI_TLX_OUT;
wire [ID_W-1:0]  ARID_AXI_TLX_OUT;
wire [31:0] ARADDR_AXI_TLX_OUT;
wire [7:0]  ARLEN_AXI_TLX_OUT;
wire [2:0]  ARSIZE_AXI_TLX_OUT;
wire [1:0]  ARBURST_AXI_TLX_OUT;
wire        ARLOCK_AXI_TLX_OUT;
wire [3:0]  ARCACHE_AXI_TLX_OUT;
wire [2:0]  ARPROT_AXI_TLX_OUT;
wire        ARVALID_AXI_TLX_OUT;
wire        ARREADY_AXI_TLX_OUT;
wire [ID_W-1:0]  RID_AXI_TLX_OUT;
wire [31:0] RDATA_AXI_TLX_OUT;
wire [1:0]  RRESP_AXI_TLX_OUT;
wire        RLAST_AXI_TLX_OUT;
wire        RVALID_AXI_TLX_OUT;
wire        RREADY_AXI_TLX_OUT;
// - PVT (silicon lifetime monitoring) APB wires
wire [31:0] PADDR_APB_PVT;
wire [31:0] PWDATA_APB_PVT;
wire        PWRITE_APB_PVT;
wire [2:0]  PPROT_APB_PVT;
wire [3:0]  PSTRB_APB_PVT;
wire        PENABLE_APB_PVT;
wire        PSELx_APB_PVT;
wire [31:0] PRDATA_APB_PVT;
wire        PSLVERR_APB_PVT;
wire        PREADY_APB_PVT;
// - Chiplet In AXI port
wire [ID_W-1:0]  AWID_AXI_CHIPLET_IN;
wire [31:0] AWADDR_AXI_CHIPLET_IN;
wire [7:0]  AWLEN_AXI_CHIPLET_IN;
wire [2:0]  AWSIZE_AXI_CHIPLET_IN;
wire [1:0]  AWBURST_AXI_CHIPLET_IN;
wire        AWLOCK_AXI_CHIPLET_IN;
wire [3:0]  AWCACHE_AXI_CHIPLET_IN;
wire [2:0]  AWPROT_AXI_CHIPLET_IN;
wire        AWVALID_AXI_CHIPLET_IN;
wire        AWREADY_AXI_CHIPLET_IN;
wire [31:0] WDATA_AXI_CHIPLET_IN;
wire [3:0]  WSTRB_AXI_CHIPLET_IN;
wire        WLAST_AXI_CHIPLET_IN;
wire        WVALID_AXI_CHIPLET_IN;
wire        WREADY_AXI_CHIPLET_IN;
wire [ID_W-1:0]  BID_AXI_CHIPLET_IN;
wire [1:0]  BRESP_AXI_CHIPLET_IN;
wire        BVALID_AXI_CHIPLET_IN;
wire        BREADY_AXI_CHIPLET_IN;
wire [ID_W-1:0]  ARID_AXI_CHIPLET_IN;
wire [31:0] ARADDR_AXI_CHIPLET_IN;
wire [7:0]  ARLEN_AXI_CHIPLET_IN;
wire [2:0]  ARSIZE_AXI_CHIPLET_IN;
wire [1:0]  ARBURST_AXI_CHIPLET_IN;
wire        ARLOCK_AXI_CHIPLET_IN;
wire [3:0]  ARCACHE_AXI_CHIPLET_IN;
wire [2:0]  ARPROT_AXI_CHIPLET_IN;
wire        ARVALID_AXI_CHIPLET_IN;
wire        ARREADY_AXI_CHIPLET_IN;
wire [ID_W-1:0]  RID_AXI_CHIPLET_IN;
wire [31:0] RDATA_AXI_CHIPLET_IN;
wire [1:0]  RRESP_AXI_CHIPLET_IN;
wire        RLAST_AXI_CHIPLET_IN;
wire        RVALID_AXI_CHIPLET_IN;
wire        RREADY_AXI_CHIPLET_IN;

// Thin links internal wires
TLX_AXI_stream #(.DATA_WIDTH(16)) TLX_IN_data_rev_pl();
TLX_AXI_stream #(.DATA_WIDTH(3)) TLX_IN_flow_rev_pl();

nic400_cd_pl_rev_M1_m_tlx_tlx_sram_chiplet u_cd_pl_rev_M1_m_tlx(
    // (tdata out)
    .tvalid_pl_rev_m1_m_tlx_m_stream(TLX_IN_data_rev.tvalid),
    .tready_pl_rev_m1_m_tlx_m_stream(TLX_IN_data_rev.tready),
    .tdata_pl_rev_m1_m_tlx_m_stream(TLX_IN_data_rev.tdata),

    // (tdata in)
    .tvalid_pl_rev_m1_m_tlx_s_stream(TLX_IN_data_rev_pl.tvalid),
    .tready_pl_rev_m1_m_tlx_s_stream(TLX_IN_data_rev_pl.tready),
    .tdata_pl_rev_m1_m_tlx_s_stream(TLX_IN_data_rev_pl.tdata),

    // (tdata out)
    .tvalid_pl_rev_m1_m_tlx_m_flow(TLX_IN_flow_rev.tvalid),
    .tready_pl_rev_m1_m_tlx_m_flow(TLX_IN_flow_rev.tready),
    .tdata_pl_rev_m1_m_tlx_m_flow(TLX_IN_flow_rev.tdata),

    // (tdata in)
    .tvalid_pl_rev_m1_m_tlx_s_flow(TLX_IN_flow_rev_pl.tvalid),
    .tready_pl_rev_m1_m_tlx_s_flow(TLX_IN_flow_rev_pl.tready),
    .tdata_pl_rev_m1_m_tlx_s_flow(TLX_IN_flow_rev_pl.tdata),

    .pl_rev_M1_m_tlxclk(SYS_CLK),
    .pl_rev_M1_m_tlxresetn(aRESETn)
);

assign IN_REV_CLK = SYS_CLK; 
assign OUT_FWD_CLK = SYS_CLK;
nic400_master_pwr_M1_m_tlx_tlx_sram_chiplet u_master_pwr_M1_m_tlx(
    // AXI manager power
    .awid_m1_m_m(AWID_AXI_CHIPLET_IN),
    .awaddr_m1_m_m(AWADDR_AXI_CHIPLET_IN),
    .awlen_m1_m_m(AWLEN_AXI_CHIPLET_IN),
    .awsize_m1_m_m(AWSIZE_AXI_CHIPLET_IN),
    .awburst_m1_m_m(AWBURST_AXI_CHIPLET_IN),
    .awlock_m1_m_m(AWLOCK_AXI_CHIPLET_IN),
    .awcache_m1_m_m(AWCACHE_AXI_CHIPLET_IN),
    .awprot_m1_m_m(AWPROT_AXI_CHIPLET_IN),
    .awvalid_m1_m_m(AWVALID_AXI_CHIPLET_IN),
    .awready_m1_m_m(AWREADY_AXI_CHIPLET_IN),
    .wdata_m1_m_m(WDATA_AXI_CHIPLET_IN),
    .wstrb_m1_m_m(WSTRB_AXI_CHIPLET_IN),
    .wlast_m1_m_m(WLAST_AXI_CHIPLET_IN),
    .wvalid_m1_m_m(WVALID_AXI_CHIPLET_IN),
    .wready_m1_m_m(WREADY_AXI_CHIPLET_IN),
    .bid_m1_m_m(BID_AXI_CHIPLET_IN),
    .bresp_m1_m_m(BRESP_AXI_CHIPLET_IN),
    .bvalid_m1_m_m(BVALID_AXI_CHIPLET_IN),
    .bready_m1_m_m(BREADY_AXI_CHIPLET_IN),
    .arid_m1_m_m(ARID_AXI_CHIPLET_IN),
    .araddr_m1_m_m(ARADDR_AXI_CHIPLET_IN),
    .arlen_m1_m_m(ARLEN_AXI_CHIPLET_IN),
    .arsize_m1_m_m(ARSIZE_AXI_CHIPLET_IN),
    .arburst_m1_m_m(ARBURST_AXI_CHIPLET_IN),
    .arlock_m1_m_m(ARLOCK_AXI_CHIPLET_IN),
    .arcache_m1_m_m(ARCACHE_AXI_CHIPLET_IN),
    .arprot_m1_m_m(ARPROT_AXI_CHIPLET_IN),
    .arvalid_m1_m_m(ARVALID_AXI_CHIPLET_IN),
    .arready_m1_m_m(ARREADY_AXI_CHIPLET_IN),
    .rid_m1_m_m(RID_AXI_CHIPLET_IN),
    .rdata_m1_m_m(RDATA_AXI_CHIPLET_IN),
    .rresp_m1_m_m(RRESP_AXI_CHIPLET_IN),
    .rlast_m1_m_m(RLAST_AXI_CHIPLET_IN),
    .rvalid_m1_m_m(RVALID_AXI_CHIPLET_IN),
    .rready_m1_m_m(RREADY_AXI_CHIPLET_IN),

    // Thin links data reverse to PL (tdata out)
    .tvalid_m1_m_tlx_tlx_m_to_pl_rev_data(TLX_IN_data_rev_pl.tvalid),
    .tready_m1_m_tlx_tlx_m_to_pl_rev_data(TLX_IN_data_rev_pl.tready),
    .tdata_m1_m_tlx_tlx_m_to_pl_rev_data(TLX_IN_data_rev_pl.tdata),

    // Thin links flow reverse to PL  (tdata out)
    .tvalid_m1_m_tlx_tlx_m_to_pl_rev_flow(TLX_IN_flow_rev_pl.tvalid),
    .tready_m1_m_tlx_tlx_m_to_pl_rev_flow(TLX_IN_flow_rev_pl.tready),
    .tdata_m1_m_tlx_tlx_m_to_pl_rev_flow(TLX_IN_flow_rev_pl.tdata),

    // Thin links data forward from PL (tdata in)
    .tvalid_m1_m_tlx_pl_fwd_to_dl_fwd_flow(TLX_IN_flow_fwd.tvalid),
    .tready_m1_m_tlx_pl_fwd_to_dl_fwd_flow(TLX_IN_flow_fwd.tready),
    .tdata_m1_m_tlx_pl_fwd_to_dl_fwd_flow(TLX_IN_flow_fwd.tdata),
  
    // Thin links flow forward from PL (tdata in)
    .tvalid_m1_m_tlx_pl_fwd_to_dl_fwd_data(TLX_IN_data_fwd.tvalid),
    .tready_m1_m_tlx_pl_fwd_to_dl_fwd_data(TLX_IN_data_fwd.tready),
    .tdata_m1_m_tlx_pl_fwd_to_dl_fwd_data(TLX_IN_data_fwd.tdata),

    // Master clock
    .clk_mclk(SYS_CLK),
    .clk_mresetn(aRESETn),
    // DL clock
    .dl_fwd_M1_m_tlxclk(IN_FWD_CLK),
    .dl_fwd_M1_m_tlxresetn(DL_FWD_RESETn)
);

// Address select logic
wire [31:0] AWADDR_AXI_CHIPLET_IN_i;
wire [31:0] ARADDR_AXI_CHIPLET_IN_i;

assign AWADDR_AXI_CHIPLET_IN_i = (AWADDR_AXI_CHIPLET_IN[20+:N_ADDR_SEL_BITS]==addr_sel[0+:N_ADDR_SEL_BITS]) ? {AWADDR_AXI_CHIPLET_IN[31:(20+N_ADDR_SEL_BITS)],{N_ADDR_SEL_BITS{1'b0}},AWADDR_AXI_CHIPLET_IN[19:0]} : AWADDR_AXI_CHIPLET_IN;
assign ARADDR_AXI_CHIPLET_IN_i = (ARADDR_AXI_CHIPLET_IN[20+:N_ADDR_SEL_BITS]==addr_sel[0+:N_ADDR_SEL_BITS]) ? {ARADDR_AXI_CHIPLET_IN[31:(20+N_ADDR_SEL_BITS)],{N_ADDR_SEL_BITS{1'b0}},ARADDR_AXI_CHIPLET_IN[19:0]} : ARADDR_AXI_CHIPLET_IN;

nic400_sram_chiplet u_nic400_sram_chiplet(
    .AWID_AXI_SRAM_0(AWID_AXI_SRAM_0),
    .AWADDR_AXI_SRAM_0(AWADDR_AXI_SRAM_0),
    .AWLEN_AXI_SRAM_0(AWLEN_AXI_SRAM_0),
    .AWSIZE_AXI_SRAM_0(AWSIZE_AXI_SRAM_0),
    .AWBURST_AXI_SRAM_0(AWBURST_AXI_SRAM_0),
    .AWLOCK_AXI_SRAM_0(AWLOCK_AXI_SRAM_0),
    .AWCACHE_AXI_SRAM_0(AWCACHE_AXI_SRAM_0),
    .AWPROT_AXI_SRAM_0(AWPROT_AXI_SRAM_0),
    .AWVALID_AXI_SRAM_0(AWVALID_AXI_SRAM_0),
    .AWREADY_AXI_SRAM_0(AWREADY_AXI_SRAM_0),
    .WDATA_AXI_SRAM_0(WDATA_AXI_SRAM_0),
    .WSTRB_AXI_SRAM_0(WSTRB_AXI_SRAM_0),
    .WLAST_AXI_SRAM_0(WLAST_AXI_SRAM_0),
    .WVALID_AXI_SRAM_0(WVALID_AXI_SRAM_0),
    .WREADY_AXI_SRAM_0(WREADY_AXI_SRAM_0),
    .BID_AXI_SRAM_0(BID_AXI_SRAM_0),
    .BRESP_AXI_SRAM_0(BRESP_AXI_SRAM_0),
    .BVALID_AXI_SRAM_0(BVALID_AXI_SRAM_0),
    .BREADY_AXI_SRAM_0(BREADY_AXI_SRAM_0),
    .ARID_AXI_SRAM_0(ARID_AXI_SRAM_0),
    .ARADDR_AXI_SRAM_0(ARADDR_AXI_SRAM_0),
    .ARLEN_AXI_SRAM_0(ARLEN_AXI_SRAM_0),
    .ARSIZE_AXI_SRAM_0(ARSIZE_AXI_SRAM_0),
    .ARBURST_AXI_SRAM_0(ARBURST_AXI_SRAM_0),
    .ARLOCK_AXI_SRAM_0(ARLOCK_AXI_SRAM_0),
    .ARCACHE_AXI_SRAM_0(ARCACHE_AXI_SRAM_0),
    .ARPROT_AXI_SRAM_0(ARPROT_AXI_SRAM_0),
    .ARVALID_AXI_SRAM_0(ARVALID_AXI_SRAM_0),
    .ARREADY_AXI_SRAM_0(ARREADY_AXI_SRAM_0),
    .RID_AXI_SRAM_0(RID_AXI_SRAM_0),
    .RDATA_AXI_SRAM_0(RDATA_AXI_SRAM_0),
    .RRESP_AXI_SRAM_0(RRESP_AXI_SRAM_0),
    .RLAST_AXI_SRAM_0(RLAST_AXI_SRAM_0),
    .RVALID_AXI_SRAM_0(RVALID_AXI_SRAM_0),
    .RREADY_AXI_SRAM_0(RREADY_AXI_SRAM_0),

    .AWID_AXI_SRAM_1(AWID_AXI_SRAM_1),
    .AWADDR_AXI_SRAM_1(AWADDR_AXI_SRAM_1),
    .AWLEN_AXI_SRAM_1(AWLEN_AXI_SRAM_1),
    .AWSIZE_AXI_SRAM_1(AWSIZE_AXI_SRAM_1),
    .AWBURST_AXI_SRAM_1(AWBURST_AXI_SRAM_1),
    .AWLOCK_AXI_SRAM_1(AWLOCK_AXI_SRAM_1),
    .AWCACHE_AXI_SRAM_1(AWCACHE_AXI_SRAM_1),
    .AWPROT_AXI_SRAM_1(AWPROT_AXI_SRAM_1),
    .AWVALID_AXI_SRAM_1(AWVALID_AXI_SRAM_1),
    .AWREADY_AXI_SRAM_1(AWREADY_AXI_SRAM_1),
    .WDATA_AXI_SRAM_1(WDATA_AXI_SRAM_1),
    .WSTRB_AXI_SRAM_1(WSTRB_AXI_SRAM_1),
    .WLAST_AXI_SRAM_1(WLAST_AXI_SRAM_1),
    .WVALID_AXI_SRAM_1(WVALID_AXI_SRAM_1),
    .WREADY_AXI_SRAM_1(WREADY_AXI_SRAM_1),
    .BID_AXI_SRAM_1(BID_AXI_SRAM_1),
    .BRESP_AXI_SRAM_1(BRESP_AXI_SRAM_1),
    .BVALID_AXI_SRAM_1(BVALID_AXI_SRAM_1),
    .BREADY_AXI_SRAM_1(BREADY_AXI_SRAM_1),
    .ARID_AXI_SRAM_1(ARID_AXI_SRAM_1),
    .ARADDR_AXI_SRAM_1(ARADDR_AXI_SRAM_1),
    .ARLEN_AXI_SRAM_1(ARLEN_AXI_SRAM_1),
    .ARSIZE_AXI_SRAM_1(ARSIZE_AXI_SRAM_1),
    .ARBURST_AXI_SRAM_1(ARBURST_AXI_SRAM_1),
    .ARLOCK_AXI_SRAM_1(ARLOCK_AXI_SRAM_1),
    .ARCACHE_AXI_SRAM_1(ARCACHE_AXI_SRAM_1),
    .ARPROT_AXI_SRAM_1(ARPROT_AXI_SRAM_1),
    .ARVALID_AXI_SRAM_1(ARVALID_AXI_SRAM_1),
    .ARREADY_AXI_SRAM_1(ARREADY_AXI_SRAM_1),
    .RID_AXI_SRAM_1(RID_AXI_SRAM_1),
    .RDATA_AXI_SRAM_1(RDATA_AXI_SRAM_1),
    .RRESP_AXI_SRAM_1(RRESP_AXI_SRAM_1),
    .RLAST_AXI_SRAM_1(RLAST_AXI_SRAM_1),
    .RVALID_AXI_SRAM_1(RVALID_AXI_SRAM_1),
    .RREADY_AXI_SRAM_1(RREADY_AXI_SRAM_1),

    .AWID_AXI_SRAM_2(AWID_AXI_SRAM_2),
    .AWADDR_AXI_SRAM_2(AWADDR_AXI_SRAM_2),
    .AWLEN_AXI_SRAM_2(AWLEN_AXI_SRAM_2),
    .AWSIZE_AXI_SRAM_2(AWSIZE_AXI_SRAM_2),
    .AWBURST_AXI_SRAM_2(AWBURST_AXI_SRAM_2),
    .AWLOCK_AXI_SRAM_2(AWLOCK_AXI_SRAM_2),
    .AWCACHE_AXI_SRAM_2(AWCACHE_AXI_SRAM_2),
    .AWPROT_AXI_SRAM_2(AWPROT_AXI_SRAM_2),
    .AWVALID_AXI_SRAM_2(AWVALID_AXI_SRAM_2),
    .AWREADY_AXI_SRAM_2(AWREADY_AXI_SRAM_2),
    .WDATA_AXI_SRAM_2(WDATA_AXI_SRAM_2),
    .WSTRB_AXI_SRAM_2(WSTRB_AXI_SRAM_2),
    .WLAST_AXI_SRAM_2(WLAST_AXI_SRAM_2),
    .WVALID_AXI_SRAM_2(WVALID_AXI_SRAM_2),
    .WREADY_AXI_SRAM_2(WREADY_AXI_SRAM_2),
    .BID_AXI_SRAM_2(BID_AXI_SRAM_2),
    .BRESP_AXI_SRAM_2(BRESP_AXI_SRAM_2),
    .BVALID_AXI_SRAM_2(BVALID_AXI_SRAM_2),
    .BREADY_AXI_SRAM_2(BREADY_AXI_SRAM_2),
    .ARID_AXI_SRAM_2(ARID_AXI_SRAM_2),
    .ARADDR_AXI_SRAM_2(ARADDR_AXI_SRAM_2),
    .ARLEN_AXI_SRAM_2(ARLEN_AXI_SRAM_2),
    .ARSIZE_AXI_SRAM_2(ARSIZE_AXI_SRAM_2),
    .ARBURST_AXI_SRAM_2(ARBURST_AXI_SRAM_2),
    .ARLOCK_AXI_SRAM_2(ARLOCK_AXI_SRAM_2),
    .ARCACHE_AXI_SRAM_2(ARCACHE_AXI_SRAM_2),
    .ARPROT_AXI_SRAM_2(ARPROT_AXI_SRAM_2),
    .ARVALID_AXI_SRAM_2(ARVALID_AXI_SRAM_2),
    .ARREADY_AXI_SRAM_2(ARREADY_AXI_SRAM_2),
    .RID_AXI_SRAM_2(RID_AXI_SRAM_2),
    .RDATA_AXI_SRAM_2(RDATA_AXI_SRAM_2),
    .RRESP_AXI_SRAM_2(RRESP_AXI_SRAM_2),
    .RLAST_AXI_SRAM_2(RLAST_AXI_SRAM_2),
    .RVALID_AXI_SRAM_2(RVALID_AXI_SRAM_2),
    .RREADY_AXI_SRAM_2(RREADY_AXI_SRAM_2),

    .AWID_AXI_SRAM_3(AWID_AXI_SRAM_3),
    .AWADDR_AXI_SRAM_3(AWADDR_AXI_SRAM_3),
    .AWLEN_AXI_SRAM_3(AWLEN_AXI_SRAM_3),
    .AWSIZE_AXI_SRAM_3(AWSIZE_AXI_SRAM_3),
    .AWBURST_AXI_SRAM_3(AWBURST_AXI_SRAM_3),
    .AWLOCK_AXI_SRAM_3(AWLOCK_AXI_SRAM_3),
    .AWCACHE_AXI_SRAM_3(AWCACHE_AXI_SRAM_3),
    .AWPROT_AXI_SRAM_3(AWPROT_AXI_SRAM_3),
    .AWVALID_AXI_SRAM_3(AWVALID_AXI_SRAM_3),
    .AWREADY_AXI_SRAM_3(AWREADY_AXI_SRAM_3),
    .WDATA_AXI_SRAM_3(WDATA_AXI_SRAM_3),
    .WSTRB_AXI_SRAM_3(WSTRB_AXI_SRAM_3),
    .WLAST_AXI_SRAM_3(WLAST_AXI_SRAM_3),
    .WVALID_AXI_SRAM_3(WVALID_AXI_SRAM_3),
    .WREADY_AXI_SRAM_3(WREADY_AXI_SRAM_3),
    .BID_AXI_SRAM_3(BID_AXI_SRAM_3),
    .BRESP_AXI_SRAM_3(BRESP_AXI_SRAM_3),
    .BVALID_AXI_SRAM_3(BVALID_AXI_SRAM_3),
    .BREADY_AXI_SRAM_3(BREADY_AXI_SRAM_3),
    .ARID_AXI_SRAM_3(ARID_AXI_SRAM_3),
    .ARADDR_AXI_SRAM_3(ARADDR_AXI_SRAM_3),
    .ARLEN_AXI_SRAM_3(ARLEN_AXI_SRAM_3),
    .ARSIZE_AXI_SRAM_3(ARSIZE_AXI_SRAM_3),
    .ARBURST_AXI_SRAM_3(ARBURST_AXI_SRAM_3),
    .ARLOCK_AXI_SRAM_3(ARLOCK_AXI_SRAM_3),
    .ARCACHE_AXI_SRAM_3(ARCACHE_AXI_SRAM_3),
    .ARPROT_AXI_SRAM_3(ARPROT_AXI_SRAM_3),
    .ARVALID_AXI_SRAM_3(ARVALID_AXI_SRAM_3),
    .ARREADY_AXI_SRAM_3(ARREADY_AXI_SRAM_3),
    .RID_AXI_SRAM_3(RID_AXI_SRAM_3),
    .RDATA_AXI_SRAM_3(RDATA_AXI_SRAM_3),
    .RRESP_AXI_SRAM_3(RRESP_AXI_SRAM_3),
    .RLAST_AXI_SRAM_3(RLAST_AXI_SRAM_3),
    .RVALID_AXI_SRAM_3(RVALID_AXI_SRAM_3),
    .RREADY_AXI_SRAM_3(RREADY_AXI_SRAM_3),

    .AWID_AXI_TLX_OUT(AWID_AXI_TLX_OUT),
    .AWADDR_AXI_TLX_OUT(AWADDR_AXI_TLX_OUT),
    .AWLEN_AXI_TLX_OUT(AWLEN_AXI_TLX_OUT),
    .AWSIZE_AXI_TLX_OUT(AWSIZE_AXI_TLX_OUT),
    .AWBURST_AXI_TLX_OUT(AWBURST_AXI_TLX_OUT),
    .AWLOCK_AXI_TLX_OUT(AWLOCK_AXI_TLX_OUT),
    .AWCACHE_AXI_TLX_OUT(AWCACHE_AXI_TLX_OUT),
    .AWPROT_AXI_TLX_OUT(AWPROT_AXI_TLX_OUT),
    .AWVALID_AXI_TLX_OUT(AWVALID_AXI_TLX_OUT),
    .AWREADY_AXI_TLX_OUT(AWREADY_AXI_TLX_OUT),
    .WDATA_AXI_TLX_OUT(WDATA_AXI_TLX_OUT),
    .WSTRB_AXI_TLX_OUT(WSTRB_AXI_TLX_OUT),
    .WLAST_AXI_TLX_OUT(WLAST_AXI_TLX_OUT),
    .WVALID_AXI_TLX_OUT(WVALID_AXI_TLX_OUT),
    .WREADY_AXI_TLX_OUT(WREADY_AXI_TLX_OUT),
    .BID_AXI_TLX_OUT(BID_AXI_TLX_OUT),
    .BRESP_AXI_TLX_OUT(BRESP_AXI_TLX_OUT),
    .BVALID_AXI_TLX_OUT(BVALID_AXI_TLX_OUT),
    .BREADY_AXI_TLX_OUT(BREADY_AXI_TLX_OUT),
    .ARID_AXI_TLX_OUT(ARID_AXI_TLX_OUT),
    .ARADDR_AXI_TLX_OUT(ARADDR_AXI_TLX_OUT),
    .ARLEN_AXI_TLX_OUT(ARLEN_AXI_TLX_OUT),
    .ARSIZE_AXI_TLX_OUT(ARSIZE_AXI_TLX_OUT),
    .ARBURST_AXI_TLX_OUT(ARBURST_AXI_TLX_OUT),
    .ARLOCK_AXI_TLX_OUT(ARLOCK_AXI_TLX_OUT),
    .ARCACHE_AXI_TLX_OUT(ARCACHE_AXI_TLX_OUT),
    .ARPROT_AXI_TLX_OUT(ARPROT_AXI_TLX_OUT),
    .ARVALID_AXI_TLX_OUT(ARVALID_AXI_TLX_OUT),
    .ARREADY_AXI_TLX_OUT(ARREADY_AXI_TLX_OUT),
    .RID_AXI_TLX_OUT(RID_AXI_TLX_OUT),
    .RDATA_AXI_TLX_OUT(RDATA_AXI_TLX_OUT),
    .RRESP_AXI_TLX_OUT(RRESP_AXI_TLX_OUT),
    .RLAST_AXI_TLX_OUT(RLAST_AXI_TLX_OUT),
    .RVALID_AXI_TLX_OUT(RVALID_AXI_TLX_OUT),
    .RREADY_AXI_TLX_OUT(RREADY_AXI_TLX_OUT),  

    .PADDR_APB_PVT(PADDR_APB_PVT),
    .PWDATA_APB_PVT(PWDATA_APB_PVT),
    .PWRITE_APB_PVT(PWRITE_APB_PVT),
    .PPROT_APB_PVT(PPROT_APB_PVT),
    .PSTRB_APB_PVT(PSTRB_APB_PVT),
    .PENABLE_APB_PVT(PENABLE_APB_PVT),
    .PSELx_APB_PVT(PSELx_APB_PVT),
    .PRDATA_APB_PVT(PRDATA_APB_PVT),
    .PSLVERR_APB_PVT(PSLVERR_APB_PVT),
    .PREADY_APB_PVT(PREADY_APB_PVT),

    .AWID_AXI_CHIPLET_IN(AWID_AXI_CHIPLET_IN),
    .AWADDR_AXI_CHIPLET_IN(AWADDR_AXI_CHIPLET_IN_i),
    .AWLEN_AXI_CHIPLET_IN(AWLEN_AXI_CHIPLET_IN),
    .AWSIZE_AXI_CHIPLET_IN(AWSIZE_AXI_CHIPLET_IN),
    .AWBURST_AXI_CHIPLET_IN(AWBURST_AXI_CHIPLET_IN),
    .AWLOCK_AXI_CHIPLET_IN(AWLOCK_AXI_CHIPLET_IN),
    .AWCACHE_AXI_CHIPLET_IN(AWCACHE_AXI_CHIPLET_IN),
    .AWPROT_AXI_CHIPLET_IN(AWPROT_AXI_CHIPLET_IN),
    .AWVALID_AXI_CHIPLET_IN(AWVALID_AXI_CHIPLET_IN),
    .AWREADY_AXI_CHIPLET_IN(AWREADY_AXI_CHIPLET_IN),
    .WDATA_AXI_CHIPLET_IN(WDATA_AXI_CHIPLET_IN),
    .WSTRB_AXI_CHIPLET_IN(WSTRB_AXI_CHIPLET_IN),
    .WLAST_AXI_CHIPLET_IN(WLAST_AXI_CHIPLET_IN),
    .WVALID_AXI_CHIPLET_IN(WVALID_AXI_CHIPLET_IN),
    .WREADY_AXI_CHIPLET_IN(WREADY_AXI_CHIPLET_IN),
    .BID_AXI_CHIPLET_IN(BID_AXI_CHIPLET_IN),
    .BRESP_AXI_CHIPLET_IN(BRESP_AXI_CHIPLET_IN),
    .BVALID_AXI_CHIPLET_IN(BVALID_AXI_CHIPLET_IN),
    .BREADY_AXI_CHIPLET_IN(BREADY_AXI_CHIPLET_IN),
    .ARID_AXI_CHIPLET_IN(ARID_AXI_CHIPLET_IN),
    .ARADDR_AXI_CHIPLET_IN(ARADDR_AXI_CHIPLET_IN_i),
    .ARLEN_AXI_CHIPLET_IN(ARLEN_AXI_CHIPLET_IN),
    .ARSIZE_AXI_CHIPLET_IN(ARSIZE_AXI_CHIPLET_IN),
    .ARBURST_AXI_CHIPLET_IN(ARBURST_AXI_CHIPLET_IN),
    .ARLOCK_AXI_CHIPLET_IN(ARLOCK_AXI_CHIPLET_IN),
    .ARCACHE_AXI_CHIPLET_IN(ARCACHE_AXI_CHIPLET_IN),
    .ARPROT_AXI_CHIPLET_IN(ARPROT_AXI_CHIPLET_IN),
    .ARVALID_AXI_CHIPLET_IN(ARVALID_AXI_CHIPLET_IN),
    .ARREADY_AXI_CHIPLET_IN(ARREADY_AXI_CHIPLET_IN),
    .RID_AXI_CHIPLET_IN(RID_AXI_CHIPLET_IN),
    .RDATA_AXI_CHIPLET_IN(RDATA_AXI_CHIPLET_IN),
    .RRESP_AXI_CHIPLET_IN(RRESP_AXI_CHIPLET_IN),
    .RLAST_AXI_CHIPLET_IN(RLAST_AXI_CHIPLET_IN),
    .RVALID_AXI_CHIPLET_IN(RVALID_AXI_CHIPLET_IN),
    .RREADY_AXI_CHIPLET_IN(RREADY_AXI_CHIPLET_IN), 

    .clk0clk(SYS_CLK),
    .clk0resetn(aRESETn),
    .refclk(SYS_CLK),
    .refclken(1'b1),
    .refresetn(aRESETn)
);

// Thin links output
// Thin links internal wires
TLX_AXI_stream #(.DATA_WIDTH(16)) TLX_OUT_data_fwd_pl();
TLX_AXI_stream #(.DATA_WIDTH(2)) TLX_OUT_flow_fwd_pl();

nic400_slave_pwr_M1_m_tlx_tlx_sram_chiplet u_slave_pwd_M1_m_tlx(
    .awid_m1_m_s(AWID_AXI_TLX_OUT),
    .awaddr_m1_m_s(AWADDR_AXI_TLX_OUT),
    .awlen_m1_m_s(AWLEN_AXI_TLX_OUT),
    .awsize_m1_m_s(AWSIZE_AXI_TLX_OUT),
    .awburst_m1_m_s(AWBURST_AXI_TLX_OUT),
    .awlock_m1_m_s(AWLOCK_AXI_TLX_OUT),
    .awcache_m1_m_s(AWCACHE_AXI_TLX_OUT),
    .awprot_m1_m_s(AWPROT_AXI_TLX_OUT),
    .awvalid_m1_m_s(AWVALID_AXI_TLX_OUT),
    .awready_m1_m_s(AWREADY_AXI_TLX_OUT),
    .wdata_m1_m_s(WDATA_AXI_TLX_OUT),
    .wstrb_m1_m_s(WSTRB_AXI_TLX_OUT),
    .wlast_m1_m_s(WLAST_AXI_TLX_OUT),
    .wvalid_m1_m_s(WVALID_AXI_TLX_OUT),
    .wready_m1_m_s(WREADY_AXI_TLX_OUT),
    .bid_m1_m_s(BID_AXI_TLX_OUT),
    .bresp_m1_m_s(BRESP_AXI_TLX_OUT),
    .bvalid_m1_m_s(BVALID_AXI_TLX_OUT),
    .bready_m1_m_s(BREADY_AXI_TLX_OUT),
    .arid_m1_m_s(ARID_AXI_TLX_OUT),
    .araddr_m1_m_s(ARADDR_AXI_TLX_OUT),
    .arlen_m1_m_s(ARLEN_AXI_TLX_OUT),
    .arsize_m1_m_s(ARSIZE_AXI_TLX_OUT),
    .arburst_m1_m_s(ARBURST_AXI_TLX_OUT),
    .arlock_m1_m_s(ARLOCK_AXI_TLX_OUT),
    .arcache_m1_m_s(ARCACHE_AXI_TLX_OUT),
    .arprot_m1_m_s(ARPROT_AXI_TLX_OUT),
    .arvalid_m1_m_s(ARVALID_AXI_TLX_OUT),
    .arready_m1_m_s(ARREADY_AXI_TLX_OUT),
    .rid_m1_m_s(RID_AXI_TLX_OUT),
    .rdata_m1_m_s(RDATA_AXI_TLX_OUT),
    .rresp_m1_m_s(RRESP_AXI_TLX_OUT),
    .rlast_m1_m_s(RLAST_AXI_TLX_OUT),
    .rvalid_m1_m_s(RVALID_AXI_TLX_OUT),
    .rready_m1_m_s(RREADY_AXI_TLX_OUT),

    // Axi Stream master (tdata out) data stream
    .tvalid_m1_m_tlx_fwd_ib_axi_stream(TLX_OUT_data_fwd_pl.tvalid),
    .tready_m1_m_tlx_fwd_ib_axi_stream(TLX_OUT_data_fwd_pl.tready),
    .tdata_m1_m_tlx_fwd_ib_axi_stream(TLX_OUT_data_fwd_pl.tdata),

    // Axi Stream master (tdata out) data flow
    .tvalid_m1_m_tlx_fwd_ib_flow(TLX_OUT_flow_fwd_pl.tvalid),
    .tready_m1_m_tlx_fwd_ib_flow(TLX_OUT_flow_fwd_pl.tready),
    .tdata_m1_m_tlx_fwd_ib_flow(TLX_OUT_flow_fwd_pl.tdata),

    // Axi Stream slave (tdata in) rev flow from SRAM chiplet PL
    .tvalid_m1_m_tlx_pl_rev_to_dl_rev_flow(TLX_OUT_flow_rev.tvalid),
    .tready_m1_m_tlx_pl_rev_to_dl_rev_flow(TLX_OUT_flow_rev.tready),
    .tdata_m1_m_tlx_pl_rev_to_dl_rev_flow(TLX_OUT_flow_rev.tdata),

    // Axi stream slave (tdata in) rev data from SRAM chiplet PL
    .tvalid_m1_m_tlx_pl_rev_to_dl_rev_data(TLX_OUT_data_rev.tvalid),
    .tready_m1_m_tlx_pl_rev_to_dl_rev_data(TLX_OUT_data_rev.tready),
    .tdata_m1_m_tlx_pl_rev_to_dl_rev_data(TLX_OUT_data_rev.tdata),

    .clk_sclk(SYS_CLK),
    .clk_sresetn(aRESETn),
    .dl_rev_M1_m_tlxclk(OUT_REV_CLK),
    .dl_rev_M1_m_tlxresetn(aRESETn)
);

nic400_cd_pl_fwd_M1_m_tlx_tlx_sram_chiplet u_pl_fwd_M1_m_tlx_m(
    // axi sream master (tdata out)
    .tvalid_pl_fwd_m1_m_tlx_m_stream(TLX_OUT_data_fwd.tvalid),
    .tready_pl_fwd_m1_m_tlx_m_stream(TLX_OUT_data_fwd.tready),
    .tdata_pl_fwd_m1_m_tlx_m_stream(TLX_OUT_data_fwd.tdata),

    // axi stream slave (tdata in)
    .tvalid_pl_fwd_m1_m_tlx_s_stream(TLX_OUT_data_fwd_pl.tvalid),
    .tready_pl_fwd_m1_m_tlx_s_stream(TLX_OUT_data_fwd_pl.tready),
    .tdata_pl_fwd_m1_m_tlx_s_stream(TLX_OUT_data_fwd_pl.tdata),

    // axi stream master (tdata out)
    .tvalid_pl_fwd_m1_m_tlx_m_flow(TLX_OUT_flow_fwd.tvalid),
    .tready_pl_fwd_m1_m_tlx_m_flow(TLX_OUT_flow_fwd.tready),
    .tdata_pl_fwd_m1_m_tlx_m_flow(TLX_OUT_flow_fwd.tdata),

    // axi stream slave (tdata in)
    .tvalid_pl_fwd_m1_m_tlx_s_flow(TLX_OUT_flow_fwd_pl.tvalid),
    .tready_pl_fwd_m1_m_tlx_s_flow(TLX_OUT_flow_fwd_pl.tready),
    .tdata_pl_fwd_m1_m_tlx_s_flow(TLX_OUT_flow_fwd_pl.tdata),

    .pl_fwd_M1_m_tlxclk(SYS_CLK),
    .pl_fwd_M1_m_tlxresetn(aRESETn)

);

wire PLL_LOCK;
sram_chiplet_apb_subsystem u_sram_chiplet_apb_subsystem(
    .ref_clk(REF_CLK),
    .PLL_CLK1(SYS_CLK),
    .PLL_CLK2(OUT_CLK),
    .aRESETn(aRESETn),
    .pll_lock(PLL_LOCK),
    .PADDR_APB_PVT(PADDR_APB_PVT),
    .PWDATA_APB_PVT(PWDATA_APB_PVT),
    .PWRITE_APB_PVT(PWRITE_APB_PVT),
    .PPROT_APB_PVT(PPROT_APB_PVT),
    .PSTRB_APB_PVT(PSTRB_APB_PVT),
    .PENABLE_APB_PVT(PENABLE_APB_PVT),
    .PSELx_APB_PVT(PSELx_APB_PVT),
    .PRDATA_APB_PVT(PRDATA_APB_PVT),
    .PSLVERR_APB_PVT(PSLVERR_APB_PVT),
    .PREADY_APB_PVT(PREADY_APB_PVT)
);

assign PLL_LOCK_o = PLL_LOCK & PLL_LOCK_i;

SRAM_wrapper #(.SPLIT_SRAM(0)) u_SRAM_wrapper_0(
    .ACLK(SYS_CLK),
    .ARESETn(aRESETn),

    .AWVALID(AWVALID_AXI_SRAM_0),
    .AWREADY(AWREADY_AXI_SRAM_0),
    .AWID(AWID_AXI_SRAM_0),
    .AWADDR(AWADDR_AXI_SRAM_0),
    .AWLEN(AWLEN_AXI_SRAM_0),
    .AWSIZE(AWSIZE_AXI_SRAM_0),
    .AWBURST(AWBURST_AXI_SRAM_0),
    .AWLOCK(AWLOCK_AXI_SRAM_0),
    .AWPROT(AWPROT_AXI_SRAM_0),
    .AWQOS(),
    .WVALID(WVALID_AXI_SRAM_0),
    .WREADY(WREADY_AXI_SRAM_0),
    .WDATA(WDATA_AXI_SRAM_0),
    .WSTRB(WSTRB_AXI_SRAM_0),
    .WLAST(WLAST_AXI_SRAM_0),
    .BVALID(BVALID_AXI_SRAM_0),
    .BREADY(BREADY_AXI_SRAM_0),
    .BID(BID_AXI_SRAM_0),
    .BRESP(BRESP_AXI_SRAM_0),
    .ARVALID(ARVALID_AXI_SRAM_0),
    .ARREADY(ARREADY_AXI_SRAM_0),
    .ARID(ARID_AXI_SRAM_0),
    .ARADDR(ARADDR_AXI_SRAM_0),
    .ARLEN(ARLEN_AXI_SRAM_0),
    .ARSIZE(ARSIZE_AXI_SRAM_0),
    .ARBURST(ARBURST_AXI_SRAM_0),
    .ARLOCK(ARLOCK_AXI_SRAM_0),
    .ARPROT(ARPROT_AXI_SRAM_0),
    .ARQOS(),
    .RVALID(RVALID_AXI_SRAM_0),
    .RREADY(RREADY_AXI_SRAM_0),
    .RID(RID_AXI_SRAM_0),
    .RDATA(RDATA_AXI_SRAM_0),
    .RRESP(RRESP_AXI_SRAM_0),
    .RLAST(RLAST_AXI_SRAM_0),
    .AWAKEUP(1'b1),

    .WPOISON(1'b0),
    .RPOISON(),

    .clk_qreqn(1'b1),
    .clk_qacceptn(),
    .clk_qdeny(),
    .clk_qactive(),

    .pwr_qreqn(1'b1),
    .pwr_qacceptn(),
    .pwr_qdeny(),
    .pwr_qactive(),

    .ext_gt_qreqn(1'b1),
    .ext_gt_qacceptn(),
    .cfg_gate_resp(1'b0)
);

SRAM_wrapper #(.SPLIT_SRAM(0)) u_SRAM_wrapper_1(
    .ACLK(SYS_CLK),
    .ARESETn(aRESETn),

    .AWVALID(AWVALID_AXI_SRAM_1),
    .AWREADY(AWREADY_AXI_SRAM_1),
    .AWID(AWID_AXI_SRAM_1),
    .AWADDR(AWADDR_AXI_SRAM_1),
    .AWLEN(AWLEN_AXI_SRAM_1),
    .AWSIZE(AWSIZE_AXI_SRAM_1),
    .AWBURST(AWBURST_AXI_SRAM_1),
    .AWLOCK(AWLOCK_AXI_SRAM_1),
    .AWPROT(AWPROT_AXI_SRAM_1),
    .AWQOS(),
    .WVALID(WVALID_AXI_SRAM_1),
    .WREADY(WREADY_AXI_SRAM_1),
    .WDATA(WDATA_AXI_SRAM_1),
    .WSTRB(WSTRB_AXI_SRAM_1),
    .WLAST(WLAST_AXI_SRAM_1),
    .BVALID(BVALID_AXI_SRAM_1),
    .BREADY(BREADY_AXI_SRAM_1),
    .BID(BID_AXI_SRAM_1),
    .BRESP(BRESP_AXI_SRAM_1),
    .ARVALID(ARVALID_AXI_SRAM_1),
    .ARREADY(ARREADY_AXI_SRAM_1),
    .ARID(ARID_AXI_SRAM_1),
    .ARADDR(ARADDR_AXI_SRAM_1),
    .ARLEN(ARLEN_AXI_SRAM_1),
    .ARSIZE(ARSIZE_AXI_SRAM_1),
    .ARBURST(ARBURST_AXI_SRAM_1),
    .ARLOCK(ARLOCK_AXI_SRAM_1),
    .ARPROT(ARPROT_AXI_SRAM_1),
    .ARQOS(),
    .RVALID(RVALID_AXI_SRAM_1),
    .RREADY(RREADY_AXI_SRAM_1),
    .RID(RID_AXI_SRAM_1),
    .RDATA(RDATA_AXI_SRAM_1),
    .RRESP(RRESP_AXI_SRAM_1),
    .RLAST(RLAST_AXI_SRAM_1),
    .AWAKEUP(1'b1),

    .WPOISON(1'b0),
    .RPOISON(),

    .clk_qreqn(1'b1),
    .clk_qacceptn(),
    .clk_qdeny(),
    .clk_qactive(),

    .pwr_qreqn(1'b1),
    .pwr_qacceptn(),
    .pwr_qdeny(),
    .pwr_qactive(),

    .ext_gt_qreqn(1'b1),
    .ext_gt_qacceptn(),
    .cfg_gate_resp(1'b0)
);

SRAM_wrapper #(.SPLIT_SRAM(0)) u_SRAM_wrapper_2(
    .ACLK(SYS_CLK),
    .ARESETn(aRESETn),

    .AWVALID(AWVALID_AXI_SRAM_2),
    .AWREADY(AWREADY_AXI_SRAM_2),
    .AWID(AWID_AXI_SRAM_2),
    .AWADDR(AWADDR_AXI_SRAM_2),
    .AWLEN(AWLEN_AXI_SRAM_2),
    .AWSIZE(AWSIZE_AXI_SRAM_2),
    .AWBURST(AWBURST_AXI_SRAM_2),
    .AWLOCK(AWLOCK_AXI_SRAM_2),
    .AWPROT(AWPROT_AXI_SRAM_2),
    .AWQOS(),
    .WVALID(WVALID_AXI_SRAM_2),
    .WREADY(WREADY_AXI_SRAM_2),
    .WDATA(WDATA_AXI_SRAM_2),
    .WSTRB(WSTRB_AXI_SRAM_2),
    .WLAST(WLAST_AXI_SRAM_2),
    .BVALID(BVALID_AXI_SRAM_2),
    .BREADY(BREADY_AXI_SRAM_2),
    .BID(BID_AXI_SRAM_2),
    .BRESP(BRESP_AXI_SRAM_2),
    .ARVALID(ARVALID_AXI_SRAM_2),
    .ARREADY(ARREADY_AXI_SRAM_2),
    .ARID(ARID_AXI_SRAM_2),
    .ARADDR(ARADDR_AXI_SRAM_2),
    .ARLEN(ARLEN_AXI_SRAM_2),
    .ARSIZE(ARSIZE_AXI_SRAM_2),
    .ARBURST(ARBURST_AXI_SRAM_2),
    .ARLOCK(ARLOCK_AXI_SRAM_2),
    .ARPROT(ARPROT_AXI_SRAM_2),
    .ARQOS(),
    .RVALID(RVALID_AXI_SRAM_2),
    .RREADY(RREADY_AXI_SRAM_2),
    .RID(RID_AXI_SRAM_2),
    .RDATA(RDATA_AXI_SRAM_2),
    .RRESP(RRESP_AXI_SRAM_2),
    .RLAST(RLAST_AXI_SRAM_2),
    .AWAKEUP(1'b1),

    .WPOISON(1'b0),
    .RPOISON(),

    .clk_qreqn(1'b1),
    .clk_qacceptn(),
    .clk_qdeny(),
    .clk_qactive(),

    .pwr_qreqn(1'b1),
    .pwr_qacceptn(),
    .pwr_qdeny(),
    .pwr_qactive(),

    .ext_gt_qreqn(1'b1),
    .ext_gt_qacceptn(),
    .cfg_gate_resp(1'b0)
);

SRAM_wrapper #(.SPLIT_SRAM(1)) u_SRAM_wrapper_3(
    .ACLK(SYS_CLK),
    .ARESETn(aRESETn),

    .AWVALID(AWVALID_AXI_SRAM_3),
    .AWREADY(AWREADY_AXI_SRAM_3),
    .AWID(AWID_AXI_SRAM_3),
    .AWADDR(AWADDR_AXI_SRAM_3),
    .AWLEN(AWLEN_AXI_SRAM_3),
    .AWSIZE(AWSIZE_AXI_SRAM_3),
    .AWBURST(AWBURST_AXI_SRAM_3),
    .AWLOCK(AWLOCK_AXI_SRAM_3),
    .AWPROT(AWPROT_AXI_SRAM_3),
    .AWQOS(),
    .WVALID(WVALID_AXI_SRAM_3),
    .WREADY(WREADY_AXI_SRAM_3),
    .WDATA(WDATA_AXI_SRAM_3),
    .WSTRB(WSTRB_AXI_SRAM_3),
    .WLAST(WLAST_AXI_SRAM_3),
    .BVALID(BVALID_AXI_SRAM_3),
    .BREADY(BREADY_AXI_SRAM_3),
    .BID(BID_AXI_SRAM_3),
    .BRESP(BRESP_AXI_SRAM_3),
    .ARVALID(ARVALID_AXI_SRAM_3),
    .ARREADY(ARREADY_AXI_SRAM_3),
    .ARID(ARID_AXI_SRAM_3),
    .ARADDR(ARADDR_AXI_SRAM_3),
    .ARLEN(ARLEN_AXI_SRAM_3),
    .ARSIZE(ARSIZE_AXI_SRAM_3),
    .ARBURST(ARBURST_AXI_SRAM_3),
    .ARLOCK(ARLOCK_AXI_SRAM_3),
    .ARPROT(ARPROT_AXI_SRAM_3),
    .ARQOS(),
    .RVALID(RVALID_AXI_SRAM_3),
    .RREADY(RREADY_AXI_SRAM_3),
    .RID(RID_AXI_SRAM_3),
    .RDATA(RDATA_AXI_SRAM_3),
    .RRESP(RRESP_AXI_SRAM_3),
    .RLAST(RLAST_AXI_SRAM_3),
    .AWAKEUP(1'b1),

    .WPOISON(1'b0),
    .RPOISON(),

    .clk_qreqn(1'b1),
    .clk_qacceptn(),
    .clk_qdeny(),
    .clk_qactive(),

    .pwr_qreqn(1'b1),
    .pwr_qacceptn(),
    .pwr_qdeny(),
    .pwr_qactive(),

    .ext_gt_qreqn(1'b1),
    .ext_gt_qacceptn(),
    .cfg_gate_resp(1'b0)
);

endmodule
