//-----------------------------------------------------------------------------
// Top pad level of SRAM chiplet for TSMC 28nm HPC+
// A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
//
// Contributors
//
// Daniel Newbrook (d.newbrook@soton.ac.uk)
//
// Copyright (C) 2021-4, SoC Labs (www.soclabs.org)
//-----------------------------------------------------------------------------

`include "tlx_interfaces.sv"
// Total pins = 106 (excluding power)

module SRAM_chiplet(
`ifdef POWER_PINS
    inout wire      VDDIO,
    inout wire      VSSIO,
    inout wire      VDD,
    inout wire      VSS,
`endif 
    input  wire         PAD_REF_CLK,
    input  wire         PAD_aRESETn,
    input  wire         PAD_DL_FWD_RESETn,
    input  wire         PAD_IN_FWD_CLK,
    output wire         PAD_IN_REV_CLK,
    output wire         PAD_OUT_FWD_CLK,
    input  wire         PAD_OUT_REV_CLK,
    output wire         PAD_PLL_LOCK_o,
    input  wire         PAD_PLL_LOCK_i,

    // TLX_IN_data_rev_0 tlx out
    output wire [15:0]  PAD_TLX_IN_data_rev_0_tdata, 
    output wire         PAD_TLX_IN_data_rev_0_tvalid,
    input  wire         PAD_TLX_IN_data_rev_0_tready,

    //TLX_IN_flow_rev_0 tlx out
    output wire [2:0]   PAD_TLX_IN_flow_rev_0_tdata, 
    output wire         PAD_TLX_IN_flow_rev_0_tvalid,
    input  wire         PAD_TLX_IN_flow_rev_0_tready,

    //tlx in
    input  wire [1:0]   PAD_TLX_IN_flow_fwd_0_tdata,
    input  wire         PAD_TLX_IN_flow_fwd_0_tvalid,
    output wire         PAD_TLX_IN_flow_fwd_0_tready,

    //tlx in
    input  wire [15:0]  PAD_TLX_IN_data_fwd_0_tdata, 
    input  wire         PAD_TLX_IN_data_fwd_0_tvalid,
    output wire         PAD_TLX_IN_data_fwd_0_tready,

    //tlx in
    input  wire [15:0]  PAD_TLX_data_rev_1_tdata, 
    input  wire         PAD_TLX_data_rev_1_tvalid,
    output wire         PAD_TLX_data_rev_1_tready,

    //tlx in
    input  wire [2:0]   PAD_TLX_flow_rev_1_tdata, 
    input  wire         PAD_TLX_flow_rev_1_tvalid,
    output wire         PAD_TLX_flow_rev_1_tready,

    // tlx out
    output wire [1:0]   PAD_TLX_flow_fwd_1_tdata, 
    output wire         PAD_TLX_flow_fwd_1_tvalid,
    input  wire         PAD_TLX_flow_fwd_1_tready,

    // tlx out
    output wire [15:0]  PAD_TLX_data_fwd_1_tdata,
    output wire         PAD_TLX_data_fwd_1_tvalid,
    input  wire         PAD_TLX_data_fwd_1_tready,

    input  wire [3:0]   PAD_addr_sel
);

// Thin links from NIC_TB to SRAM_chiplet_0
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_IN_data_rev_0();
TLX_AXI_stream #(.DATA_WIDTH(3))    TLX_IN_flow_rev_0();
TLX_AXI_stream #(.DATA_WIDTH(2))    TLX_IN_flow_fwd_0();
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_IN_data_fwd_0();

// Thin links from SRAM_chiplet_0 to SRAM_chiplet_1
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_data_rev_1();
TLX_AXI_stream #(.DATA_WIDTH(3))    TLX_flow_rev_1();
TLX_AXI_stream #(.DATA_WIDTH(2))    TLX_flow_fwd_1();
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_data_fwd_1();

    // TLX_IN_data_rev_0 tlx out
assign PAD_TLX_IN_data_rev_0_tdata  = TLX_IN_data_rev_0.tdata;
assign PAD_TLX_IN_data_rev_0_tvalid = TLX_IN_data_rev_0.tvalid;
assign TLX_IN_data_rev_0.tready = PAD_TLX_IN_data_rev_0_tready;

    //TLX_IN_flow_rev_0 tlx out
assign PAD_TLX_IN_flow_rev_0_tdata  = TLX_IN_flow_rev_0.tdata;
assign PAD_TLX_IN_flow_rev_0_tvalid = TLX_IN_flow_rev_0.tvalid;
assign TLX_IN_flow_rev_0.tready  = PAD_TLX_IN_flow_rev_0_tready;

    //tlx in
assign TLX_IN_flow_fwd_0.tdata  = PAD_TLX_IN_flow_fwd_0_tdata;
assign TLX_IN_flow_fwd_0.tvalid = PAD_TLX_IN_flow_fwd_0_tvalid;
assign PAD_TLX_IN_flow_fwd_0_tready = TLX_IN_flow_fwd_0.tready;

    //tlx in
assign TLX_IN_data_fwd_0.tdata  = PAD_TLX_IN_data_fwd_0_tdata;
assign TLX_IN_data_fwd_0.tvalid = PAD_TLX_IN_data_fwd_0_tvalid;
assign PAD_TLX_IN_data_fwd_0_tready = TLX_IN_data_fwd_0.tready;

    //tlx in
assign TLX_data_rev_1.tdata     = PAD_TLX_data_rev_1_tdata;
assign TLX_data_rev_1.tvalid    = PAD_TLX_data_rev_1_tvalid;
assign PAD_TLX_data_rev_1_tready    = TLX_data_rev_1.tready;

    //tlx in
assign TLX_flow_rev_1.tdata     = PAD_TLX_flow_rev_1_tdata;
assign TLX_flow_rev_1.tvalid    = PAD_TLX_flow_rev_1_tvalid;
assign PAD_TLX_flow_rev_1_tready    = TLX_flow_rev_1.tready;

    // tlx out
assign PAD_TLX_flow_fwd_1_tdata     = TLX_flow_fwd_1.tdata;
assign PAD_TLX_flow_fwd_1_tvalid    = TLX_flow_fwd_1.tvalid;
assign TLX_flow_fwd_1.tready    = PAD_TLX_flow_fwd_1_tready;

    // tlx out
assign PAD_TLX_data_fwd_1_tdata     = TLX_data_fwd_1.tdata;
assign PAD_TLX_data_fwd_1_tvalid    = TLX_data_fwd_1.tvalid;
assign TLX_data_fwd_1.tready    = PAD_TLX_data_fwd_1_tready;



top_sram_chiplet #(.N_ADDR_SEL_BITS(4)) u_top_sram_chiplet (
    .REF_CLK(PAD_REF_CLK),
    .aRESETn(PAD_aRESETn),
    .DL_FWD_RESETn(PAD_DL_FWD_RESETn),

    .IN_FWD_CLK(PAD_IN_FWD_CLK),
    .IN_REV_CLK(PAD_IN_REV_CLK),
    .OUT_FWD_CLK(PAD_OUT_FWD_CLK),
    .OUT_REV_CLK(PAD_OUT_REV_CLK),
    .PLL_LOCK_o(PAD_PLL_LOCK_o),
    .PLL_LOCK_i(PAD_PLL_LOCK_i),

    // Thin Links In
    .TLX_IN_data_rev(TLX_IN_data_rev_0),
    .TLX_IN_flow_rev(TLX_IN_flow_rev_0),
    .TLX_IN_flow_fwd(TLX_IN_flow_fwd_0),
    .TLX_IN_data_fwd(TLX_IN_data_fwd_0),
    // Thin Links Out
    .TLX_OUT_data_rev(TLX_data_rev_1),
    .TLX_OUT_flow_rev(TLX_flow_rev_1),
    .TLX_OUT_flow_fwd(TLX_flow_fwd_1),
    .TLX_OUT_data_fwd(TLX_data_fwd_1),
    
    // Address Select pins
    .addr_sel(PAD_addr_sel)
);

endmodule