//-----------------------------------------------------------------------------
// Top pad level of SRAM chiplet for TSMC 28nm HPC+
// A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
//
// Contributors
//
// Daniel Newbrook (d.newbrook@soton.ac.uk)
//
// Copyright (C) 2021-4, SoC Labs (www.soclabs.org)
//-----------------------------------------------------------------------------

`include "tlx_interfaces.sv"
// Total pins = 106 (excluding power)

module SRAM_chiplet(
`ifdef POWER_PINS
    inout wire      VDDIO,
    inout wire      VSSIO,
    inout wire      VDD,
    inout wire      VSS,
`endif 
    input  wire         PAD_REF_CLK,
    input  wire         PAD_aRESETn,
    input  wire         PAD_DL_FWD_RESETn,
    input  wire         PAD_IN_FWD_CLK,
    output wire         PAD_IN_REV_CLK,
    output wire         PAD_OUT_FWD_CLK,
    input  wire         PAD_OUT_REV_CLK,
    output wire         PAD_PLL_LOCK_o,
    input  wire         PAD_PLL_LOCK_i,

    input  wire         PAD_VPREF,

    // TLX_IN_data_rev_0 tlx out
    output wire [15:0]  PAD_TLX_IN_data_rev_0_tdata, 
    output wire         PAD_TLX_IN_data_rev_0_tvalid,
    input  wire         PAD_TLX_IN_data_rev_0_tready,

    //TLX_IN_flow_rev_0 tlx out
    output wire [2:0]   PAD_TLX_IN_flow_rev_0_tdata, 
    output wire         PAD_TLX_IN_flow_rev_0_tvalid,
    input  wire         PAD_TLX_IN_flow_rev_0_tready,

    //tlx in
    input  wire [1:0]   PAD_TLX_IN_flow_fwd_0_tdata,
    input  wire         PAD_TLX_IN_flow_fwd_0_tvalid,
    output wire         PAD_TLX_IN_flow_fwd_0_tready,

    //tlx in
    input  wire [15:0]  PAD_TLX_IN_data_fwd_0_tdata, 
    input  wire         PAD_TLX_IN_data_fwd_0_tvalid,
    output wire         PAD_TLX_IN_data_fwd_0_tready,

    //tlx in
    input  wire [15:0]  PAD_TLX_data_rev_1_tdata, 
    input  wire         PAD_TLX_data_rev_1_tvalid,
    output wire         PAD_TLX_data_rev_1_tready,

    //tlx in
    input  wire [2:0]   PAD_TLX_flow_rev_1_tdata, 
    input  wire         PAD_TLX_flow_rev_1_tvalid,
    output wire         PAD_TLX_flow_rev_1_tready,

    // tlx out
    output wire [1:0]   PAD_TLX_flow_fwd_1_tdata, 
    output wire         PAD_TLX_flow_fwd_1_tvalid,
    input  wire         PAD_TLX_flow_fwd_1_tready,

    // tlx out
    output wire [15:0]  PAD_TLX_data_fwd_1_tdata,
    output wire         PAD_TLX_data_fwd_1_tvalid,
    input  wire         PAD_TLX_data_fwd_1_tready,

    input  wire [3:0]   PAD_addr_sel
);
wire         REF_CLK;
wire         aRESETn;
wire         DL_FWD_RESETn;
wire         IN_FWD_CLK;
wire         IN_REV_CLK;
wire         OUT_FWD_CLK;
wire         OUT_REV_CLK;
wire         PLL_LOCK_o;
wire         PLL_LOCK_i;
wire [3:0]   addr_sel;

wire [15:0]  TLX_IN_data_rev_0_tdata; 
wire         TLX_IN_data_rev_0_tvalid;
wire         TLX_IN_data_rev_0_tready;
wire [2:0]   TLX_IN_flow_rev_0_tdata; 
wire         TLX_IN_flow_rev_0_tvalid;
wire         TLX_IN_flow_rev_0_tready;
wire [1:0]   TLX_IN_flow_fwd_0_tdata;
wire         TLX_IN_flow_fwd_0_tvalid;
wire         TLX_IN_flow_fwd_0_tready;
wire [15:0]  TLX_IN_data_fwd_0_tdata; 
wire         TLX_IN_data_fwd_0_tvalid;
wire         TLX_IN_data_fwd_0_tready;
wire [15:0]  TLX_data_rev_1_tdata; 
wire         TLX_data_rev_1_tvalid;
wire         TLX_data_rev_1_tready;
wire [2:0]   TLX_flow_rev_1_tdata; 
wire         TLX_flow_rev_1_tvalid;
wire         TLX_flow_rev_1_tready;
wire [1:0]   TLX_flow_fwd_1_tdata; 
wire         TLX_flow_fwd_1_tvalid;
wire         TLX_flow_fwd_1_tready;
wire [15:0]  TLX_data_fwd_1_tdata;
wire         TLX_data_fwd_1_tvalid;
wire         TLX_data_fwd_1_tready;

// Thin links from NIC_TB to SRAM_chiplet_0
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_IN_data_rev_0();
TLX_AXI_stream #(.DATA_WIDTH(3))    TLX_IN_flow_rev_0();
TLX_AXI_stream #(.DATA_WIDTH(2))    TLX_IN_flow_fwd_0();
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_IN_data_fwd_0();

// Thin links from SRAM_chiplet_0 to SRAM_chiplet_1
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_data_rev_1();
TLX_AXI_stream #(.DATA_WIDTH(3))    TLX_flow_rev_1();
TLX_AXI_stream #(.DATA_WIDTH(2))    TLX_flow_fwd_1();
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_data_fwd_1();

    // TLX_IN_data_rev_0 tlx out
assign TLX_IN_data_rev_0_tdata  = TLX_IN_data_rev_0.tdata;
assign TLX_IN_data_rev_0_tvalid = TLX_IN_data_rev_0.tvalid;
assign TLX_IN_data_rev_0.tready = TLX_IN_data_rev_0_tready;

    //TLX_IN_flow_rev_0 tlx out
assign TLX_IN_flow_rev_0_tdata  = TLX_IN_flow_rev_0.tdata;
assign TLX_IN_flow_rev_0_tvalid = TLX_IN_flow_rev_0.tvalid;
assign TLX_IN_flow_rev_0.tready  = TLX_IN_flow_rev_0_tready;

    //tlx in
assign TLX_IN_flow_fwd_0.tdata  = TLX_IN_flow_fwd_0_tdata;
assign TLX_IN_flow_fwd_0.tvalid = TLX_IN_flow_fwd_0_tvalid;
assign TLX_IN_flow_fwd_0_tready = TLX_IN_flow_fwd_0.tready;

    //tlx in
assign TLX_IN_data_fwd_0.tdata  = TLX_IN_data_fwd_0_tdata;
assign TLX_IN_data_fwd_0.tvalid = TLX_IN_data_fwd_0_tvalid;
assign TLX_IN_data_fwd_0_tready = TLX_IN_data_fwd_0.tready;

    //tlx in
assign TLX_data_rev_1.tdata     = TLX_data_rev_1_tdata;
assign TLX_data_rev_1.tvalid    = TLX_data_rev_1_tvalid;
assign TLX_data_rev_1_tready    = TLX_data_rev_1.tready;

    //tlx in
assign TLX_flow_rev_1.tdata     = TLX_flow_rev_1_tdata;
assign TLX_flow_rev_1.tvalid    = TLX_flow_rev_1_tvalid;
assign TLX_flow_rev_1_tready    = TLX_flow_rev_1.tready;

    // tlx out
assign TLX_flow_fwd_1_tdata     = TLX_flow_fwd_1.tdata;
assign TLX_flow_fwd_1_tvalid    = TLX_flow_fwd_1.tvalid;
assign TLX_flow_fwd_1.tready    = TLX_flow_fwd_1_tready;

    // tlx out
assign TLX_data_fwd_1_tdata     = TLX_data_fwd_1.tdata;
assign TLX_data_fwd_1_tvalid    = TLX_data_fwd_1.tvalid;
assign TLX_data_fwd_1.tready    = TLX_data_fwd_1_tready;


genvar i;
generate 
    for(i=0;i<16;i=i+1) begin  : TLX_data_tdata
        PDDW04DGZ_V_G u_TLX_IN_data_rev_tdata(
            .OEN(1'b0),
            .REN(1'b1),
            .I(TLX_IN_data_rev_0_tdata[i]),
            .C(),
            .PAD(PAD_TLX_IN_data_rev_0_tdata[i])
        );
        PDDW04DGZ_V_G u_TLX_IN_data_fwd_tdata(
            .OEN(1'b1),
            .REN(1'b1),
            .I(1'b0),
            .C(TLX_IN_data_fwd_0_tdata[i]),
            .PAD(PAD_TLX_IN_data_fwd_0_tdata[i])
        );
        PDDW04DGZ_V_G u_TLX_data_rev_1_tdata(
            .OEN(1'b1),
            .REN(1'b1),
            .I(1'b0),
            .C(TLX_data_rev_1_tdata[i]),
            .PAD(PAD_TLX_data_rev_1_tdata[i])
        );
        PDDW04DGZ_V_G u_TLX_data_fwd_1_tdata(
            .OEN(1'b0),
            .REN(1'b1),
            .I(TLX_data_fwd_1_tdata[i]),
            .C(),
            .PAD(PAD_TLX_data_fwd_1_tdata[i])
        );
    end
endgenerate

// Thin Links Outputs
PDDW04DGZ_V_G u_PAD_TLX_IN_data_rev_0_tvalid( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_data_rev_0_tvalid),
    .C(),
    .PAD(PAD_TLX_IN_data_rev_0_tvalid)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_rev_0_tdata_0 ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_flow_rev_0_tdata[0]),
    .C(),
    .PAD(PAD_TLX_IN_flow_rev_0_tdata[0])
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_rev_0_tdata_1 ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_flow_rev_0_tdata[1]),
    .C(),
    .PAD(PAD_TLX_IN_flow_rev_0_tdata[1])
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_rev_0_tdata_2 ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_flow_rev_0_tdata[2]),
    .C(),
    .PAD(PAD_TLX_IN_flow_rev_0_tdata[2])
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_rev_0_tvalid( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_flow_rev_0_tvalid),
    .C(),
    .PAD(PAD_TLX_IN_flow_rev_0_tvalid)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_fwd_0_tready ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_flow_fwd_0_tready),
    .C(),
    .PAD(PAD_TLX_IN_flow_fwd_0_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_data_fwd_0_tready ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_IN_data_fwd_0_tready),
    .C(),
    .PAD(PAD_TLX_IN_data_fwd_0_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_data_rev_1_tready ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_data_rev_1_tready),
    .C(),
    .PAD(PAD_TLX_data_rev_1_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_flow_rev_1_tready ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_flow_rev_1_tready),
    .C(),
    .PAD(PAD_TLX_flow_rev_1_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_flow_fwd_1_tdata_0 ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_flow_fwd_1_tdata[0]),
    .C(),
    .PAD(PAD_TLX_flow_fwd_1_tdata[0])
);

PDDW04DGZ_V_G u_PAD_TLX_flow_fwd_1_tdata_1 ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_flow_fwd_1_tdata[1]),
    .C(),
    .PAD(PAD_TLX_flow_fwd_1_tdata[1])
);

PDDW04DGZ_V_G u_PAD_TLX_flow_fwd_1_tvalid ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_flow_fwd_1_tvalid),
    .C(),
    .PAD(PAD_TLX_flow_fwd_1_tvalid)
);


PDDW04DGZ_V_G u_PAD_TLX_data_fwd_1_tvalid ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(TLX_data_fwd_1_tvalid),
    .C(),
    .PAD(PAD_TLX_data_fwd_1_tvalid)
);

PDDW04DGZ_V_G u_PAD_IN_REV_CLK ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(IN_REV_CLK),
    .C(),
    .PAD(PAD_IN_REV_CLK)
);

PDDW04DGZ_V_G u_PAD_OUT_FWD_CLK ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(OUT_FWD_CLK),
    .C(),
    .PAD(PAD_OUT_FWD_CLK)
);

PDDW04DGZ_V_G u_PAD_PLL_LOCK_o ( 
    .OEN(1'b0),
    .REN(1'b1),
    .I(PLL_LOCK_o),
    .C(),
    .PAD(PAD_PLL_LOCK_o)
);

// inputs
PDDW04DGZ_V_G u_PAD_REF_CLK(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(REF_CLK),
    .PAD(PAD_REF_CLK)
);

PDDW04DGZ_V_G u_PAD_aRESETn(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(aRESETn),
    .PAD(PAD_aRESETn)
);

PDDW04DGZ_V_G u_PAD_DL_FWD_RESETn(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(DL_FWD_RESETn),
    .PAD(PAD_DL_FWD_RESETn)
);

PDDW04DGZ_V_G u_PAD_IN_FWD_CLK(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(IN_FWD_CLK),
    .PAD(PAD_IN_FWD_CLK)
);

PDDW04DGZ_V_G u_PAD_OUT_REV_CLK(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(OUT_REV_CLK),
    .PAD(PAD_OUT_REV_CLK)
);

PDDW04DGZ_V_G u_PAD_PLL_LOCK_i(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(PLL_LOCK_i),
    .PAD(PAD_PLL_LOCK_i)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_data_rev_0_tready(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_IN_data_rev_0_tready),
    .PAD(PAD_TLX_IN_data_rev_0_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_rev_0_tready(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_IN_flow_rev_0_tready),
    .PAD(PAD_TLX_IN_flow_rev_0_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_fwd_0_tdata_0(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_IN_flow_fwd_0_tdata[0]),
    .PAD(PAD_TLX_IN_flow_fwd_0_tdata[0])
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_fwd_0_tdata_1(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_IN_flow_fwd_0_tdata[1]),
    .PAD(PAD_TLX_IN_flow_fwd_0_tdata[1])
);

PDDW04DGZ_V_G u_PAD_TLX_IN_flow_fwd_0_tvalid(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_IN_flow_fwd_0_tvalid),
    .PAD(PAD_TLX_IN_flow_fwd_0_tvalid)
);

PDDW04DGZ_V_G u_PAD_TLX_IN_data_fwd_0_tvalid(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_IN_data_fwd_0_tvalid),
    .PAD(PAD_TLX_IN_data_fwd_0_tvalid)
);

PDDW04DGZ_V_G u_PAD_TLX_data_rev_1_tvalid(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_data_rev_1_tvalid),
    .PAD(PAD_TLX_data_rev_1_tvalid)
);

PDDW04DGZ_V_G u_PAD_TLX_flow_rev_1_tdata_0(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_flow_rev_1_tdata[0]),
    .PAD(PAD_TLX_flow_rev_1_tdata[0])
);

PDDW04DGZ_V_G u_PAD_TLX_flow_rev_1_tdata_1(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_flow_rev_1_tdata[1]),
    .PAD(PAD_TLX_flow_rev_1_tdata[1])
);

PDDW04DGZ_V_G u_PAD_TLX_flow_rev_1_tdata_2(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_flow_rev_1_tdata[2]),
    .PAD(PAD_TLX_flow_rev_1_tdata[2])
);

PDDW04DGZ_V_G u_PAD_TLX_flow_rev_1_tvalid(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_flow_rev_1_tvalid),
    .PAD(PAD_TLX_flow_rev_1_tvalid)
);

PDDW04DGZ_V_G u_PAD_TLX_flow_fwd_1_tready(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_flow_fwd_1_tready),
    .PAD(PAD_TLX_flow_fwd_1_tready)
);

PDDW04DGZ_V_G u_PAD_TLX_data_fwd_1_tready(
    .OEN(1'b1),
    .REN(1'b1),
    .I(1'b0),
    .C(TLX_data_fwd_1_tready),
    .PAD(PAD_TLX_data_fwd_1_tready)
);

genvar j;
generate  
    for(j=0;j<4;j=j+1) begin : gen_addr_sel
        PDDW04DGZ_V_G u_PAD_addr_sel(
            .OEN(1'b1),
            .REN(1'b1),
            .I(1'b0),
            .C(addr_sel[j]),
            .PAD(PAD_addr_sel[j])
        );
    end
endgenerate

// Power 
 PVDD1DGZ_H_G u_PAD_VDD_top(

 );
 PVDD1DGZ_H_G u_PAD_VDD_left(

 );
 PVDD1DGZ_H_G u_PAD_VDD_right(

 );
 PVDD1DGZ_H_G u_PAD_VDD_top_right(

 );

 PVDD1DGZ_H_G u_PAD_VDD_bottom(

 );

 PVDD2POC_H_G u_PAD_VDDIO_POC_left(

 );

 PVDD2DGZ_H_G u_PAD_VDDIO_right(

 );

 PVDD2DGZ_H_G u_PAD_VDDIO_top(

 );

 PVDD2DGZ_H_G u_PAD_VDDIO_bottom(

 );



 PVSS2DGZ_H_G u_PAD_VSSIO_left(

 );

 PVSS2DGZ_H_G u_PAD_VSSIO_right(

 );

 PVSS2DGZ_H_G u_PAD_VSSIO_top(

 );

 PVSS2DGZ_H_G u_PAD_VSSIO_bottom(

 );


 PVSS1DGZ_H_G u_PAD_VSS_top(

 );
 PVSS1DGZ_H_G u_PAD_VSS_left(

 );
 PVSS1DGZ_H_G u_PAD_VSS_right(

 );
 PVSS1DGZ_H_G u_PAD_VSS_bottom(

 );

 PVDD1ANA_H_G u_PAD_AVDD_left(

 );
 PVDD2ANA_H_G u_PAD_AVDDHV_left(

 );
 PVSS1ANA_H_G u_PAD_AGND_left(

 );
 PVSS2ANA_H_G u_PAD_AGNDHV_left(

 );

top_sram_chiplet #(.N_ADDR_SEL_BITS(4)) u_top_sram_chiplet (
    .REF_CLK(REF_CLK),
    .aRESETn(aRESETn),
    .DL_FWD_RESETn(DL_FWD_RESETn),

    .IN_FWD_CLK(IN_FWD_CLK),
    .IN_REV_CLK(IN_REV_CLK),
    .OUT_FWD_CLK(OUT_FWD_CLK),
    .OUT_REV_CLK(OUT_REV_CLK),
    .PLL_LOCK_o(PLL_LOCK_o),
    .PLL_LOCK_i(PLL_LOCK_i),

    // Thin Links In
    .TLX_IN_data_rev(TLX_IN_data_rev_0),
    .TLX_IN_flow_rev(TLX_IN_flow_rev_0),
    .TLX_IN_flow_fwd(TLX_IN_flow_fwd_0),
    .TLX_IN_data_fwd(TLX_IN_data_fwd_0),
    // Thin Links Out
    .TLX_OUT_data_rev(TLX_data_rev_1),
    .TLX_OUT_flow_rev(TLX_flow_rev_1),
    .TLX_OUT_flow_fwd(TLX_flow_fwd_1),
    .TLX_OUT_data_fwd(TLX_data_fwd_1),
    
    // Address Select pins
    .addr_sel(addr_sel)
);

endmodule