`timescale 1ns/1ps

`include "tlx_interfaces.sv"

module sram_chiplet_cocotb(

    input  wire [11:0]   cocotb_awid,
    input  wire [31:0]  cocotb_awaddr,
    input  wire [7:0]   cocotb_awlen,
    input  wire [2:0]   cocotb_awsize,
    input  wire [1:0]   cocotb_awburst,
    input  wire         cocotb_awlock,
    input  wire [3:0]   cocotb_awcache,
    input  wire [2:0]   cocotb_awprot,
    input  wire         cocotb_awvalid,
    output wire         cocotb_awready,
    input  wire [31:0]  cocotb_wdata,
    input  wire [3:0]   cocotb_wstrb,
    input  wire         cocotb_wlast,
    input  wire         cocotb_wvalid,
    output wire         cocotb_wready,
    output wire [11:0]   cocotb_bid,
    output wire [1:0]   cocotb_bresp,
    output wire         cocotb_bvalid,
    input  wire         cocotb_bready,
    input  wire [11:0]   cocotb_arid,
    input  wire [31:0]  cocotb_araddr,
    input  wire [7:0]   cocotb_arlen,
    input  wire [2:0]   cocotb_arsize,
    input  wire [1:0]   cocotb_arburst,
    input  wire         cocotb_arlock,
    input  wire [3:0]   cocotb_arcache,
    input  wire [2:0]   cocotb_arprot,
    input  wire         cocotb_arvalid,
    output wire         cocotb_arready,
    output wire [11:0]   cocotb_rid,
    output wire [31:0]  cocotb_rdata,
    output wire [1:0]   cocotb_rresp,
    output wire         cocotb_rlast,
    output wire         cocotb_rvalid,
    input  wire         cocotb_rready  
);


wire         clk_in;
wire         aresetn;

reg clock_q;
reg [32:0] shifter;

initial begin 
    clock_q <= 1'b0;
    shifter <= 32'd0;
    #40 clock_q <= 1'b1;
end

always @(clock_q)
    #1 clock_q <= !clock_q;

assign clk_in = clock_q;

always @(posedge clock_q) begin 
    if(!(&shifter))
        shifter <= {shifter[30:0],1'b1};
end

assign aresetn = shifter[31];

parameter N_ADDR_SEL_BITS=4;
parameter N_CHIPLETS=2;

wire DL_REV_CLK[N_CHIPLETS+1];
wire DL_FWD_CLK[N_CHIPLETS+1];

// Thin links array
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_data_rev[N_CHIPLETS+1]();
TLX_AXI_stream #(.DATA_WIDTH(3))    TLX_flow_rev[N_CHIPLETS+1]();
TLX_AXI_stream #(.DATA_WIDTH(2))    TLX_flow_fwd[N_CHIPLETS+1]();
TLX_AXI_stream #(.DATA_WIDTH(16))   TLX_data_fwd[N_CHIPLETS+1]();

wire [11:0]  AWID_AXI_CHIPLET_OUT;
wire [31:0] AWADDR_AXI_CHIPLET_OUT;
wire [7:0]  AWLEN_AXI_CHIPLET_OUT;
wire [2:0]  AWSIZE_AXI_CHIPLET_OUT;
wire [1:0]  AWBURST_AXI_CHIPLET_OUT;
wire        AWLOCK_AXI_CHIPLET_OUT;
wire [3:0]  AWCACHE_AXI_CHIPLET_OUT;
wire [2:0]  AWPROT_AXI_CHIPLET_OUT;
wire        AWVALID_AXI_CHIPLET_OUT;
wire        AWREADY_AXI_CHIPLET_OUT;
wire [31:0] WDATA_AXI_CHIPLET_OUT;
wire [3:0]  WSTRB_AXI_CHIPLET_OUT;
wire        WLAST_AXI_CHIPLET_OUT;
wire        WVALID_AXI_CHIPLET_OUT;
wire        WREADY_AXI_CHIPLET_OUT;
wire [11:0]  BID_AXI_CHIPLET_OUT;
wire [1:0]  BRESP_AXI_CHIPLET_OUT;
wire        BVALID_AXI_CHIPLET_OUT;
wire        BREADY_AXI_CHIPLET_OUT;
wire [11:0]  ARID_AXI_CHIPLET_OUT;
wire [31:0] ARADDR_AXI_CHIPLET_OUT;
wire [7:0]  ARLEN_AXI_CHIPLET_OUT;
wire [2:0]  ARSIZE_AXI_CHIPLET_OUT;
wire [1:0]  ARBURST_AXI_CHIPLET_OUT;
wire        ARLOCK_AXI_CHIPLET_OUT;
wire [3:0]  ARCACHE_AXI_CHIPLET_OUT;
wire [2:0]  ARPROT_AXI_CHIPLET_OUT;
wire        ARVALID_AXI_CHIPLET_OUT;
wire        ARREADY_AXI_CHIPLET_OUT;
wire [11:0]  RID_AXI_CHIPLET_OUT;
wire [31:0] RDATA_AXI_CHIPLET_OUT;
wire [1:0]  RRESP_AXI_CHIPLET_OUT;
wire        RLAST_AXI_CHIPLET_OUT;
wire        RVALID_AXI_CHIPLET_OUT;
wire        RREADY_AXI_CHIPLET_OUT;

wire        tvalid_m1_m_tlx_fwd_ib_axi_stream;
wire        tready_m1_m_tlx_fwd_ib_axi_stream;
wire [15:0] tdata_m1_m_tlx_fwd_ib_axi_stream;

wire        tvalid_m1_m_tlx_fwd_ib_flow;
wire        tready_m1_m_tlx_fwd_ib_flow;
wire [1:0]  tdata_m1_m_tlx_fwd_ib_flow;
nic400_tb u_nic400_tb(
    .AWID_AXI_CHIPLET_OUT(AWID_AXI_CHIPLET_OUT),
    .AWADDR_AXI_CHIPLET_OUT(AWADDR_AXI_CHIPLET_OUT),
    .AWLEN_AXI_CHIPLET_OUT(AWLEN_AXI_CHIPLET_OUT),
    .AWSIZE_AXI_CHIPLET_OUT(AWSIZE_AXI_CHIPLET_OUT),
    .AWBURST_AXI_CHIPLET_OUT(AWBURST_AXI_CHIPLET_OUT),
    .AWLOCK_AXI_CHIPLET_OUT(AWLOCK_AXI_CHIPLET_OUT),
    .AWCACHE_AXI_CHIPLET_OUT(AWCACHE_AXI_CHIPLET_OUT),
    .AWPROT_AXI_CHIPLET_OUT(AWPROT_AXI_CHIPLET_OUT),
    .AWVALID_AXI_CHIPLET_OUT(AWVALID_AXI_CHIPLET_OUT),
    .AWREADY_AXI_CHIPLET_OUT(AWREADY_AXI_CHIPLET_OUT),
    .WDATA_AXI_CHIPLET_OUT(WDATA_AXI_CHIPLET_OUT),
    .WSTRB_AXI_CHIPLET_OUT(WSTRB_AXI_CHIPLET_OUT),
    .WLAST_AXI_CHIPLET_OUT(WLAST_AXI_CHIPLET_OUT),
    .WVALID_AXI_CHIPLET_OUT(WVALID_AXI_CHIPLET_OUT),
    .WREADY_AXI_CHIPLET_OUT(WREADY_AXI_CHIPLET_OUT),
    .BID_AXI_CHIPLET_OUT(BID_AXI_CHIPLET_OUT),
    .BRESP_AXI_CHIPLET_OUT(BRESP_AXI_CHIPLET_OUT),
    .BVALID_AXI_CHIPLET_OUT(BVALID_AXI_CHIPLET_OUT),
    .BREADY_AXI_CHIPLET_OUT(BREADY_AXI_CHIPLET_OUT),
    .ARID_AXI_CHIPLET_OUT(ARID_AXI_CHIPLET_OUT),
    .ARADDR_AXI_CHIPLET_OUT(ARADDR_AXI_CHIPLET_OUT),
    .ARLEN_AXI_CHIPLET_OUT(ARLEN_AXI_CHIPLET_OUT),
    .ARSIZE_AXI_CHIPLET_OUT(ARSIZE_AXI_CHIPLET_OUT),
    .ARBURST_AXI_CHIPLET_OUT(ARBURST_AXI_CHIPLET_OUT),
    .ARLOCK_AXI_CHIPLET_OUT(ARLOCK_AXI_CHIPLET_OUT),
    .ARCACHE_AXI_CHIPLET_OUT(ARCACHE_AXI_CHIPLET_OUT),
    .ARPROT_AXI_CHIPLET_OUT(ARPROT_AXI_CHIPLET_OUT),
    .ARVALID_AXI_CHIPLET_OUT(ARVALID_AXI_CHIPLET_OUT),
    .ARREADY_AXI_CHIPLET_OUT(ARREADY_AXI_CHIPLET_OUT),
    .RID_AXI_CHIPLET_OUT(RID_AXI_CHIPLET_OUT),
    .RDATA_AXI_CHIPLET_OUT(RDATA_AXI_CHIPLET_OUT),
    .RRESP_AXI_CHIPLET_OUT(RRESP_AXI_CHIPLET_OUT),
    .RLAST_AXI_CHIPLET_OUT(RLAST_AXI_CHIPLET_OUT),
    .RVALID_AXI_CHIPLET_OUT(RVALID_AXI_CHIPLET_OUT),
    .RREADY_AXI_CHIPLET_OUT(RREADY_AXI_CHIPLET_OUT),

    .AWID_AXI_COCOTB(cocotb_awid),
    .AWADDR_AXI_COCOTB(cocotb_awaddr),
    .AWLEN_AXI_COCOTB(cocotb_awlen),
    .AWSIZE_AXI_COCOTB(cocotb_awsize),
    .AWBURST_AXI_COCOTB(cocotb_awburst),
    .AWLOCK_AXI_COCOTB(cocotb_awlock),
    .AWCACHE_AXI_COCOTB(cocotb_awcache),
    .AWPROT_AXI_COCOTB(cocotb_awprot),
    .AWVALID_AXI_COCOTB(cocotb_awvalid),
    .AWREADY_AXI_COCOTB(cocotb_awready),
    .WDATA_AXI_COCOTB(cocotb_wdata),
    .WSTRB_AXI_COCOTB(cocotb_wstrb),
    .WLAST_AXI_COCOTB(cocotb_wlast),
    .WVALID_AXI_COCOTB(cocotb_wvalid),
    .WREADY_AXI_COCOTB(cocotb_wready),
    .BID_AXI_COCOTB(cocotb_bid),
    .BRESP_AXI_COCOTB(cocotb_bresp),
    .BVALID_AXI_COCOTB(cocotb_bvalid),
    .BREADY_AXI_COCOTB(cocotb_bready),
    .ARID_AXI_COCOTB(cocotb_arid),
    .ARADDR_AXI_COCOTB(cocotb_araddr),
    .ARLEN_AXI_COCOTB(cocotb_arlen),
    .ARSIZE_AXI_COCOTB(cocotb_arsize),
    .ARBURST_AXI_COCOTB(cocotb_arburst),
    .ARLOCK_AXI_COCOTB(cocotb_arlock),
    .ARCACHE_AXI_COCOTB(cocotb_arcache),
    .ARPROT_AXI_COCOTB(cocotb_arprot),
    .ARVALID_AXI_COCOTB(cocotb_arvalid),
    .ARREADY_AXI_COCOTB(cocotb_arready),
    .RID_AXI_COCOTB(cocotb_rid),
    .RDATA_AXI_COCOTB(cocotb_rdata),
    .RRESP_AXI_COCOTB(cocotb_rresp),
    .RLAST_AXI_COCOTB(cocotb_rlast),
    .RVALID_AXI_COCOTB(cocotb_rvalid),
    .RREADY_AXI_COCOTB(cocotb_rready),

    .clk0clk(clk_in),
    .clk0resetn(aresetn)
);


nic400_slave_pwr_M1_m_tlx_tlx_sram_chiplet u_slave_pwd_M1_m_tlx(
    .awid_m1_m_s(AWID_AXI_CHIPLET_OUT),
    .awaddr_m1_m_s(AWADDR_AXI_CHIPLET_OUT),
    .awlen_m1_m_s(AWLEN_AXI_CHIPLET_OUT),
    .awsize_m1_m_s(AWSIZE_AXI_CHIPLET_OUT),
    .awburst_m1_m_s(AWBURST_AXI_CHIPLET_OUT),
    .awlock_m1_m_s(AWLOCK_AXI_CHIPLET_OUT),
    .awcache_m1_m_s(AWCACHE_AXI_CHIPLET_OUT),
    .awprot_m1_m_s(AWPROT_AXI_CHIPLET_OUT),
    .awvalid_m1_m_s(AWVALID_AXI_CHIPLET_OUT),
    .awready_m1_m_s(AWREADY_AXI_CHIPLET_OUT),
    .wdata_m1_m_s(WDATA_AXI_CHIPLET_OUT),
    .wstrb_m1_m_s(WSTRB_AXI_CHIPLET_OUT),
    .wlast_m1_m_s(WLAST_AXI_CHIPLET_OUT),
    .wvalid_m1_m_s(WVALID_AXI_CHIPLET_OUT),
    .wready_m1_m_s(WREADY_AXI_CHIPLET_OUT),
    .bid_m1_m_s(BID_AXI_CHIPLET_OUT),
    .bresp_m1_m_s(BRESP_AXI_CHIPLET_OUT),
    .bvalid_m1_m_s(BVALID_AXI_CHIPLET_OUT),
    .bready_m1_m_s(BREADY_AXI_CHIPLET_OUT),
    .arid_m1_m_s(ARID_AXI_CHIPLET_OUT),
    .araddr_m1_m_s(ARADDR_AXI_CHIPLET_OUT),
    .arlen_m1_m_s(ARLEN_AXI_CHIPLET_OUT),
    .arsize_m1_m_s(ARSIZE_AXI_CHIPLET_OUT),
    .arburst_m1_m_s(ARBURST_AXI_CHIPLET_OUT),
    .arlock_m1_m_s(ARLOCK_AXI_CHIPLET_OUT),
    .arcache_m1_m_s(ARCACHE_AXI_CHIPLET_OUT),
    .arprot_m1_m_s(ARPROT_AXI_CHIPLET_OUT),
    .arvalid_m1_m_s(ARVALID_AXI_CHIPLET_OUT),
    .arready_m1_m_s(ARREADY_AXI_CHIPLET_OUT),
    .rid_m1_m_s(RID_AXI_CHIPLET_OUT),
    .rdata_m1_m_s(RDATA_AXI_CHIPLET_OUT),
    .rresp_m1_m_s(RRESP_AXI_CHIPLET_OUT),
    .rlast_m1_m_s(RLAST_AXI_CHIPLET_OUT),
    .rvalid_m1_m_s(RVALID_AXI_CHIPLET_OUT),
    .rready_m1_m_s(RREADY_AXI_CHIPLET_OUT),

    // Axi Stream master (tdata out) data stream
    .tvalid_m1_m_tlx_fwd_ib_axi_stream(tvalid_m1_m_tlx_fwd_ib_axi_stream),
    .tready_m1_m_tlx_fwd_ib_axi_stream(tready_m1_m_tlx_fwd_ib_axi_stream),
    .tdata_m1_m_tlx_fwd_ib_axi_stream(tdata_m1_m_tlx_fwd_ib_axi_stream),

    // Axi Stream master (tdata out) data flow
    .tvalid_m1_m_tlx_fwd_ib_flow(tvalid_m1_m_tlx_fwd_ib_flow),
    .tready_m1_m_tlx_fwd_ib_flow(tready_m1_m_tlx_fwd_ib_flow),
    .tdata_m1_m_tlx_fwd_ib_flow(tdata_m1_m_tlx_fwd_ib_flow),

    // Axi Stream slave (tdata in) rev flow from SRAM chiplet PL
    .tvalid_m1_m_tlx_pl_rev_to_dl_rev_flow(TLX_flow_rev[0].tvalid),
    .tready_m1_m_tlx_pl_rev_to_dl_rev_flow(TLX_flow_rev[0].tready),
    .tdata_m1_m_tlx_pl_rev_to_dl_rev_flow(TLX_flow_rev[0].tdata),

    // Axi stream slave (tdata in) rev data from SRAM chiplet PL
    .tvalid_m1_m_tlx_pl_rev_to_dl_rev_data(TLX_data_rev[0].tvalid),
    .tready_m1_m_tlx_pl_rev_to_dl_rev_data(TLX_data_rev[0].tready),
    .tdata_m1_m_tlx_pl_rev_to_dl_rev_data(TLX_data_rev[0].tdata),

    .clk_sclk(clk_in),
    .clk_sresetn(aresetn),
    .dl_rev_M1_m_tlxclk(DL_REV_CLK[0]),
    .dl_rev_M1_m_tlxresetn(aresetn)
);

nic400_cd_pl_fwd_M1_m_tlx_tlx_sram_chiplet u_pl_fwd_M1_m_tlx_m(
    // axi sream master (tdata out)
    .tvalid_pl_fwd_m1_m_tlx_m_stream(TLX_data_fwd[0].tvalid),
    .tready_pl_fwd_m1_m_tlx_m_stream(TLX_data_fwd[0].tready),
    .tdata_pl_fwd_m1_m_tlx_m_stream(TLX_data_fwd[0].tdata),

    // axi stream slave (tdata in)
    .tvalid_pl_fwd_m1_m_tlx_s_stream(tvalid_m1_m_tlx_fwd_ib_axi_stream),
    .tready_pl_fwd_m1_m_tlx_s_stream(tready_m1_m_tlx_fwd_ib_axi_stream),
    .tdata_pl_fwd_m1_m_tlx_s_stream(tdata_m1_m_tlx_fwd_ib_axi_stream),

    // axi stream master (tdata out)
    .tvalid_pl_fwd_m1_m_tlx_m_flow(TLX_flow_fwd[0].tvalid),
    .tready_pl_fwd_m1_m_tlx_m_flow(TLX_flow_fwd[0].tready),
    .tdata_pl_fwd_m1_m_tlx_m_flow(TLX_flow_fwd[0].tdata),

    // axi stream slave (tdata in)
    .tvalid_pl_fwd_m1_m_tlx_s_flow(tvalid_m1_m_tlx_fwd_ib_flow),
    .tready_pl_fwd_m1_m_tlx_s_flow(tready_m1_m_tlx_fwd_ib_flow),
    .tdata_pl_fwd_m1_m_tlx_s_flow(tdata_m1_m_tlx_fwd_ib_flow),

    .pl_fwd_M1_m_tlxclk(clk_in),
    .pl_fwd_M1_m_tlxresetn(aresetn)
);

wire [N_CHIPLETS:0] ref_clk;
reg [1:0] clock_div;
always @(posedge clk_in or negedge aresetn) begin
    if(~aresetn)
        clock_div <= 2'b00;
    else 
        clock_div <= clock_div + 1;
end

assign ref_clk[0] = clock_div[1];

wire [N_CHIPLETS:0] PLL_LOCK;
assign DL_FWD_CLK[0] = clk_in;
initial begin 
    $sdf_annotate("/home/dwn1c21/SoC-Labs/TAPEOUT/jan2025/sram_chiplet/imp/ASIC/SRAM_CHIPLET/sram_chiplet_gate.sdf", gen_chiplets[0].u_top_sram_chiplet);
end
genvar i;
generate
    for(i=0; i<N_CHIPLETS;i=i+1) begin: gen_chiplets
        SRAM_chiplet u_top_sram_chiplet(
            .PAD_REF_CLK(ref_clk[0]),

            .PAD_aRESETn(aresetn),
            .PAD_DL_FWD_RESETn(aresetn),
            .PAD_PLL_LOCK_o(PLL_LOCK[i]),
            .PAD_PLL_LOCK_i(PLL_LOCK[i+1]),
            .PAD_IN_FWD_CLK(DL_FWD_CLK[i]),
            .PAD_IN_REV_CLK(DL_REV_CLK[i]),
            .PAD_OUT_FWD_CLK(DL_FWD_CLK[i+1]),
            .PAD_OUT_REV_CLK(DL_REV_CLK[i+1]),

            .PAD_TLX_IN_data_rev_0_tdata    (TLX_data_rev[i].tdata),
            .PAD_TLX_IN_data_rev_0_tvalid   (TLX_data_rev[i].tvalid),
            .PAD_TLX_IN_data_rev_0_tready   (TLX_data_rev[i].tready),

            .PAD_TLX_IN_flow_rev_0_tdata    (TLX_flow_rev[i].tdata),
            .PAD_TLX_IN_flow_rev_0_tvalid   (TLX_flow_rev[i].tvalid),
            .PAD_TLX_IN_flow_rev_0_tready   (TLX_flow_rev[i].tready),

            .PAD_TLX_IN_flow_fwd_0_tdata    (TLX_flow_fwd[i].tdata),
            .PAD_TLX_IN_flow_fwd_0_tvalid   (TLX_flow_fwd[i].tvalid),
            .PAD_TLX_IN_flow_fwd_0_tready   (TLX_flow_fwd[i].tready),

            .PAD_TLX_IN_data_fwd_0_tdata    (TLX_data_fwd[i].tdata),
            .PAD_TLX_IN_data_fwd_0_tvalid   (TLX_data_fwd[i].tvalid),
            .PAD_TLX_IN_data_fwd_0_tready   (TLX_data_fwd[i].tready),

            .PAD_TLX_data_rev_1_tdata   (TLX_data_rev[i+1].tdata),
            .PAD_TLX_data_rev_1_tvalid  (TLX_data_rev[i+1].tvalid),
            .PAD_TLX_data_rev_1_tready  (TLX_data_rev[i+1].tready),

            .PAD_TLX_flow_rev_1_tdata   (TLX_flow_rev[i+1].tdata),
            .PAD_TLX_flow_rev_1_tvalid  (TLX_flow_rev[i+1].tvalid),
            .PAD_TLX_flow_rev_1_tready  (TLX_flow_rev[i+1].tready),

            .PAD_TLX_flow_fwd_1_tdata   (TLX_flow_fwd[i+1].tdata),
            .PAD_TLX_flow_fwd_1_tvalid  (TLX_flow_fwd[i+1].tvalid),
            .PAD_TLX_flow_fwd_1_tready  (TLX_flow_fwd[i+1].tready),

            .PAD_TLX_data_fwd_1_tdata   (TLX_data_fwd[i+1].tdata),
            .PAD_TLX_data_fwd_1_tvalid  (TLX_data_fwd[i+1].tvalid),
            .PAD_TLX_data_fwd_1_tready  (TLX_data_fwd[i+1].tready),
            // Address select
            .PAD_addr_sel(i[N_ADDR_SEL_BITS-1:0])  
 
        );
    end
endgenerate

assign PLL_LOCK[N_CHIPLETS]=1'b1;

endmodule